/*
 * BSD Telephony Of Mexico "Tormenta" card LINUX driver, version 1.8 4/8/01
 * 
 * Working with the "Tormenta ISA" Card 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under thet erms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 * Primary Author: Jim Dixon <jim@lambdatel.com> 
 *
 * Networking and enhancements: Mark Spencer <markster@linux-support.net>
 *
 *  Version History:
 *
 *  1.9pre 4/11/01
 *  Added TOR_AUDIOMODE ioctl
 *  Added echo canceller
 *
 *  1.8  4/8/01
 *	Added FXO and FXS Kewlstart functionality. Added special conference
 *      mode functionality for pseudo-channels.
 *
 *  1.7  3/13/01
 *	Fixed typographical error regarding FCS calculation
 * 
 *  1.6  3/11/01
 *      Added Networking support
 *	Added support for zone-specific tones
 *	Moved to dynamic buffer allocation
 *	Handle exceptions
 * 
 *  1.5  1/1/01
 *      Modified tor_open function allowing for multiple entities to
 *      open the master device (/dev/tor0).
 *
 *  1.4  12/30/00
 *      Added mode NOTOPEN alarm functionality, where if a span is put in
 *      TOR_CONFIG_NOTOPEN mode and no channels are open on that span,
 *      it will send Yellow alarm and show in alarm state. Also added
 *      support for conference mute mode.
 *
 *  1.3  12/12/00
 *      Added support for FXO Loopstart and Groundstart signalling.
 *
 *  1.2  12/11/00
 *      Initial port to Linux.
 *
 *  1.1  11/7/00
 *	Updated version to correspond with release. No significant changes made.
 *
 *  1.0  11/6/00
 *	Changed memory map to correspond with the real ISA PC card.
 * 
 *  0.6  11/4/00
 *      Added TOR_TXSTATE_AFTERSTART, so that there is a pause after
 *      starting a ground-start trunk.
 *
 *  0.5  11/3/00
 *      Added CORRECT support for ESF and B8ZS (gosh, it sure does HELP to
 *      put the RECEIVER in the correct mode, too!!). Also found rather
 *      annoying bug in the Dallas 2151Q chip (or could be a board noise
 *      problem). Occasionally, accesses to the chip (reads and writes) for
 *      some random reason cause CRC errors on the transmit output, thus
 *      making ESF mode useless.
 */



/*   Driver Documentation

This is a somewhat unusual driver. Most of the work takes place in the
interrupt routine (buffer handling, board I/O, switching, conferencing, 
timing, signalling handling, etc). In addition, its not exactly a character
device, and its not exactly a block device (explained later).


INTERRUPT ROUTINE

Every 125 microseconds (8000 times a second), the Tormenta card generates
a hardware interrupt to the processor. At this time, the computer responds
to this interrupt, and does the following:

    Reads in 48 samples (1 per channel) of speech data from the T-1 channels.
    Does processing for various switching and conferencing modes (moves
	audio data around, etc).
    Adds samples to read buffers.
    Gets samples from write buffers.
    Writes out 48 samples (1 per channel) of speech data to the T-1 channels.

Every 8 interrupts (1000 times a second), the various timers are checked
for activity and time-outs. A timer that times-out has a service routine
associated with it that gets executed.

Every 512 interrupts (15.625 times a second), signalling bits are looked
at, processed, handled (by associated routines), and written as appropriate.
Also, alarm status is looked at, and handled accordingly.


BUFFERS, READING, AND WRITING

Each channel contains 2 to 8 buffers (first in, first out) for reading
and writing with the default being 2 buffers.  They are handled in the
following manner:

A buffer has 3 states: empty, being filled, and full.

In the case of the write buffers, instead of usual buffer processing, 
where a buffer is incrementally filled until full, a write buffer in
this implementation is considered full when any block (including a 
blocksize of 1) is written to it. Only 1 buffer at a time may be written
to. Therefore, the device really only should be written to at the full
current blocksize (less would not be efficient use of the buffering system).
The idea is to be able to fill (write to) a buffer, and have that one being
emptied while you fill (write to) the other buffer(s).  

In the case of the read buffers, instead of usual buffer processing, 
where a buffer is incrementally emptied until totally empty, a full read
buffer in this implementation is fully emptied when a read occurs, regardless
whether the read transfer all or part of the size of the read buffer.
Therefore, the device really only should be read from at the full current
blocklize (less would lose data). The idea is to read from a full buffer
while an other read buffer is being filled.

Both read and write calls are interrupted by an event happening (see events
below). If an event happens, the call fails with ELAST as the error (indicating
a pending event that must be read out of the event buffer).


DATA FLOW, SWITCHING, AND CONFERENCING

A channel's data normally flows from the received sample from the T-1 channel
into the read buffer, and from the write buffer to the transmitted sample
on the T-1 channel. 

This implementation allows for various modifications to the above data
flow concept (see related figures for archetecture detail). These various
"conference" modes allow for monitoring, switching, and conferencing with
many variations.


EVENT BUFFERS AND HANDLING

Every channel has an event buffer associated with it. There are a number
of occurrences that an application need to be notified of, such as
a channel going off hook (or ringing), a channel going on hook (disconnecting),
a channel winking, a carrier alarm condition, or a lack of carrier alarm
condition, kernel-based tone/signalling generation events, and conditions
associated with HDLC transmission/reception.

When each of these events occur, notification of such is inserted into
an event buffer (queue), and then can be read out of the buffer in the
order in which it (or they) happened. 

If a read or write system call is in progress, it is interrupted and
fails with an ELAST error code (indicating that there is a pending event
that must be read out of the buffer).

Events are read out of the buffer using the TOR_GETEVENT ioctl.


I/O MULTIPLEXING (aka 'select' on steriods)

For applications that need to do several I/O operations simultaneously on
a single channel, the TOR_IOMUX ioctl is available to allow this to happen
efficiently. 

You specify a mask of things that you wish to look for, such as read available,
write space availiable, write empty, or event pending.

Upon return, the mask is set to indicate which of these things took place
and caused the call to return.


DATA MODES

Several 'Clear Channel' data modes are supported. These are supported on
either a channel-by-channel basis, or 'bundled' multiple channel basis, where
several related channels' data are combined onto one single buffer. These modes
include 'Clear' mode, where 8 bit data is transparently transferred to and
from a channels (or set of channels') buffer, 'HDLC' mode, where a channel's
(or set of channels' data is converted to and from HDLC frames), and
'Net HDLC' mode, which is the same as HDLC mode, except the data goes between
the channel(s) the HDLC sender/receiver and a Linux-Native networking
interface (tornet, see below). Obviously, to use any of the 'Clear' data
modes with 8 bits, the span needs to be in a mode supporting it (B8ZS).


NETWORKING (aka 'tornet')

For data applications, the Tormenta driver can be configured to perform fast
HDLC using a fast, PATENT PENDING table based algorithm.  The fasthdlc
implementation is licensed freely under GPL, so the patent issue should
only conern you if you are interested in using the algorithm in some other
proprietary program. The patent and licensing issue does NOT affect any
userspace program that interacts with this driver.

To use the tornet device you will need the sethdlc utility.  For more
information on sethdlc see http://hq.pm.waw.pl/hdlc/


TONE ZONES (TONE GENERATION WITHIN THE DRIVER)

Tone zones are a method for enabling a driver to ask the kernel to play
certain common tones (like dialtone, busy, ringback, etc) and even DTMF
through ioctl()'s.  Tone zones are loaded at runtime by the TOR_LOAD_ZONE
ioctl.  A series of tables for the tones are passed in the ioctl, and then
when the tone is played, the kernel simply increments through each sample
of the table one at a time (and each table of the tone) to play the tone.
You can also set a default "zone" so that applications which play tones
can transparently use the tone zone of the default country.  e.g. if you
configure the default tonezone to be "fr" then the channel will generate
French dialtone, ringback, etc.  Of course, DTMF and MF(R1) tones are hard
coded in and not loaded by userspace.


IOCTL's

The 'tor' driver supports the following IOCTL's:

Get Transfer Block Size
	TOR_GET_BLOCKSIZE	int * -- location (pointer) to which to 
				    return current blocksize in integer
Set Transfer Block Size.
	TOR_SET_BLOCKSIZE	int * -- pointer to integer containing new
				    blocksize
Flush Buffer(s) and stop I/O
	TOR_FLUSH		int * -- pointer to integer mask indicating
				    what to flush 
Wait for Write to Finish
	TOR_SYNC		void

Get channel parameters
	TOR_GET_PARAMS		struct tor_params * -- pointer to struct to
				    contain current channel parameters
Set channel parameters
	TOR_SET_PARAMS		struct tor_params * -- pointer to struct which
				    contains new channel parameters
Set Hookswitch Status
	TOR_HOOK		int * -- New hookswitch state

Get Signalling Event
	TOR_GETEVENT		int * -- pointer to integer in which the 
				    event is returned
Wait for something to happen
	TOR_IOMUX		int * -- enters with pointer to integer mask
				    specifying desired things to wait for.
				    Exits after writing integer mask (at
				    same location) of what it found.

Get Span Status
	TOR_SPANSTAT		struct tor_spaninfo * -- Pointer to span
				    info structure. Enter with desired span
				    number filled in spanno (or 0 for current),
				    exits with all span infomation filled in.

Set Maintenance (Loopback) Mode
	TOR_MAINT		struct tor_maintinfo * -- Pointer to maint
				    info structure. Enter with desired span
				    number filled in spanno (or 0 for current),
				    and desired maintenance mode filled in.

Get Conference Mode
	TOR_GETCONF		struct tor_confinfo * -- Pointer to conf info
				    structure. Enter with channel number
				    filled in (0 for current). Exits with
				    conf info structure filled in.
Set Conference Mode
	TOR_SETCONF		struct tor_confinfo * -- Pointer to conf info
				    structure. Enter with channel number filled
				    in (0 for current), and conf number,
				    and conf mode.

Setup or Remove Conference Link
	TOR_CONFLINK		struct tor_confinfo * -- Pointer to conf info
				    structure. In this case, channel number
				    is filled in with the desired conference
				    number to listen to. Confn is filled in
				    with the desired conference number that
				    is to listen to the conference specified
				    in channel number. Fill-in confmode non-
				    zero to create the specified link, or
				    zero to delete (break) the specified
				    link.

Display Conference Diagnostic Information on Console
	TOR_CONFDIAG		int * -- Pointer to integer, 0 specifies all
				    conferences, non-zero specifies a 
				    particular conference only.

Get Receive and Transmit Gains (linear)
	TOR_GETGAINS		struct tor_gains * -- Pointer to tor_gains 
				    structure. Enter with channel number filled
				    in (0 for current), and returns channel,
				    and rx & tx gains (linear, not db).

Set Receive and Transmit Gains (linear)
	TOR_SETGAINS		struct tor_gains * -- Pointer to tor_gains 
				    structure. Enter with channel number filled
				    in (0 for current), and rx & tx gains
				    (linear, not db).

Set Channel Configuration
	TOR_CHANCONFIG		struct tor_chanconfig * -- Pointer to 
				     tor_chanconfig structure. Enter with 
				     channel number in chan, and signalling
				     type in sigtype.

Set T-1 Line Configurations and Start System
	TOR_STARTUP		struct tor_lineconfig * -- Pointer to
				     tor_lineconfig structure. Enter with
				     all parameters filled in for each span.

Mute conference temporarily
	TOR_CONFMUTE	int * -- Pointer to integer, 0 disables muting, non-zero
	                 enables muting of conference for this channel.  Helps in
					 dtmf detection.

Send a local tone
    TOR_SENDTONE    int * -- Plays a specific tone, or a value of -1 stops any
				 	 currently playing tone.  Tone played will be chosen from the
					 current tone zone.  Returns ENODATA if the current zone is not
					 loaded or ENOSYS if the zone is loaded but this tone does not
					 exist in the zone.
					 Values for tones are:
					 TOR_TONE_STOP - Stops any currrently playing tone
					 TOR_TONE_DIALTONE - Plays a dialtone
					 TOR_TONE_BUSY - Generates the busy cadence					 
					 TOR_TONE_RINGTONE - Generates ringback cadence
					 TOR_TONE_CONGESTION - Generates congestion (fast busy) sound
					 TOR_TONE_CALLWAIT - Generates call waiting
					 TOR_TONE_DIALRECALL - Generates special "recall" dialtone
					 TOR_TONE_RECORDTONE - Generates tone to indicate recording
					 TOR_TONE_INFO - Special Information tone ("your call cannot be completed...")
					 
					 Note that not all tones are guaranteed to be implemented so you must
					 check the return value.
					 
					 Note also that tones are turned off as soon as you write to the device.

Set numerical tone zone
	TOR_SETTONEZONE int * -- Sets the current channel's tone zone to the one specified
	
Get numerical tone zone
	TOR_GETTONEZONE int * -- Retrieves the current tone zone numerical and stores it in the
					passed integer
					
Set default tone zone
	TOR_DEFAULTZONE int * -- Sets the default tone zone for channels which do not specify one
				    to this value.
					
Load zone information
	TOR_LOADZONE	struct tor_tone_def_header * -- Provides information (including ring
					cadence and all known tones).  See tor.h for more information.
					
Free zone information
	TOR_FREEZONE	int * -- Frees the specified tone zone's kernel data structures.
	
Set buffer information
	TOR_SET_BUFINFO struct tor_bufferinfo * -- Sets the size and number of buffers, as well
					as the buffer policy (on transmitter or receiver independently) as follows:
						POLICY_IMMEDIATE: As soon as a single buffer is full, begin transmission
						                  or receive end.
						POLICY_WHEN_FULL: Only begin transmission or reception when all buffers
						                  are full.
										  																 
Get buffer information
	TOR_GET_BUFINFO struct tor_bufferinfo * -- Retreives current buffer settings as specified above
	
Set dialing parameters
	TOR_SET_DIALPARAMS struct tor_dialparams * -- Sets dialing parameters for kernel-generated
					   dialing.  Specifically:
					   
					   	mfv1_tonelen: Then length (in milliseconds) of an MF (version 1) tone
						dtmf_tonelen: Then length (in milliseconds) of a DTMF tone
						
						Note that the KP tone is 5/3's the length of the specified mfv1_tonelen
						
Dial Operation
	TOR_DIAL          struct tor_dialoperation * -- Performs one of the following operations:
	                  TOR_DIAL_OP_APPEND: Append data in dialstr to current transmission list.
					  TOR_DIAL_OP_REPLACE: Replace current transmission list with dialstr
					  TOR_DIAL_OP_CANCEL: Immediately cancel any dialing operation.
					  The dialstr field contains 0-9, A-D, *, and # as tones and 'M' or 'T' to
					  specify MF or TouchTone for the digits following.  The maximum buffer
					  length is 256 outstanding characters.

Set or clear audio mode on clear channels
	TOR_AUDIOMODE     int -- Enables or disables audio mode on a clear channel.   If you want
				      your CLEAR channels to participate in audio conferences and have transmit
					  and receive gain applied, call this ioctl with a non-zero argument, otherwise
					  with a zero argument.

Shutdown card 
	TOR_SHUTDOWN	  (no parameters) -- Shuts down the tormenta card, disables the interrupt
					  handler, and should place all spans in alarm.

For detailed information of the IOCTL's and their various arguments and
parameters, see the 'tor.h' header file.

Note: A TOR_CHANCONFIG ioctl must be done for each channel (both physical
and virtual), and then a TOR_STARTUP must be done to start the system.

Possible error codes returned from IOCTL's:

EINVAL -- Invalid or out of range argument specified.

EEXIST -- Specified conference link already exists.

ENOSPC -- No space available to additional conference link.

ENOENT -- No such conference link (can not locate specified link).


HARDWARE ARCHETECTURE

The hardware is a 16 bit memory-mapped ISA bus card. It uses both 8 bit
and 16 bit memory cycles, specified by a 16 bit enable line in the control
register. The Mitel MT8920 STPA chips are accessed (in parallel) with
16 bit memory accesses, and everything else on the card uses 8 bit memory
accesses.

There is a control register (output) that contains the following:

Timing genrator (MT8941 DPLL) mode control (MS0,MS1).
Interrupt enable.
Outbit (the output bit that is brought out for exernal measurement).
Receive clock control/enable for both T-1 chips.
16 bit memory access enable.

Reading from the status (control) register contains the following:

Interrupt pending.


The MT8920 STPA chips must be accessed in 16 bit mode, where the 16
bit memory access enable bit on the control register is true. All other
accesses must be done in 8 bit mode, where the 16 bit memory access enable
bit is false.

The timing (syncronization source) is controlled by a combination of the
mode select bits on the MT8941 DPLL, and the receive clock control/enable
bits for the two T-1 chips.

The two Dallas T-1 chips reside at 2 address blocks, one for the data
memory (with the ALE line not asserted), and one for the address memory
(with the ALE line asserted). Reading and/or writing from/to the Dallas
T-1 chips requires two operations. First is a write of the internal
chip register address to the address memory, then a read/write from/to
the data memory. This is required because of the Dallas chip's pseudo-
multiplexed data/address bus. See the Dallas DS2151Q chip documentation for
more information.

*/

#ifdef STANDALONE_TORMENTA
#include "tor.h"
#else
#include <linux/tor.h>
#endif

#if NTOR > 0
#include <linux/config.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/version.h>
#include <linux/slab.h>	 /* for kmalloc */
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/mm.h>    /* for verify_area */
#include <linux/errno.h> /* for -EBUSY */
#include <linux/poll.h>
#include <linux/ioport.h> /* for autoirq stuff */
#include <linux/delay.h>
#include <asm/segment.h> /* for put_user_byte */
#include <asm/io.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
#include <asm/spinlock.h>
#else
#include <linux/spinlock.h>
#endif
#include <linux/netdevice.h>
#ifdef CONFIG_TORMENTA_NET
#include <net/syncppp.h>
#include <linux/hdlc.h>
#endif
#include <linux/kmod.h>
#include <linux/ppp_defs.h>
#include "fasthdlc.h"
#ifdef CONFIG_TORMENTA_ECHOCANCEL
#include "mec.h"
#endif

/* Board address offsets (specified in word (not byte) offsets) */
#define	DDATA 0			/* Data I/O Register */
#define	DADDR 0x100		/* Dallas Card Address Reg., 0x200 in byte offset higher */
#define	CTLREG 0x100		/* Control/Status Reg., 0x200 in byte offset */

/* Control register bits */
#define	MASTERCLOCK 0		/* Value for Master Clock */
#define	OUTBIT 8		/* Status output bit (for external measurements) */
#define	INTENA 4		/* Interrupt enable bit */
#define	ENA16 0x80		/* 16 bit bus cycle enable bit */

/* size of event buffer */
#define	TOR_MAX_EVENTSIZE	64	/* 64 events max */

/* There is a table like this in the PPP driver, too */

static 
__u16 fcstab[256] =
{
	0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf,
	0x8c48, 0x9dc1, 0xaf5a, 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7,
	0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c, 0x75b7, 0x643e,
	0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876,
	0x2102, 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd,
	0xad4a, 0xbcc3, 0x8e58, 0x9fd1, 0xeb6e, 0xfae7, 0xc87c, 0xd9f5,
	0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5, 0x453c,
	0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974,
	0x4204, 0x538d, 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb,
	0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868, 0x99e1, 0xab7a, 0xbaf3,
	0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
	0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72,
	0x6306, 0x728f, 0x4014, 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9,
	0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3, 0x8a78, 0x9bf1,
	0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x0738,
	0xffcf, 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70,
	0x8408, 0x9581, 0xa71a, 0xb693, 0xc22c, 0xd3a5, 0xe13e, 0xf0b7,
	0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76, 0x7cff,
	0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036,
	0x18c1, 0x0948, 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e,
	0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e, 0xf2a7, 0xc03c, 0xd1b5,
	0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
	0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134,
	0x39c3, 0x284a, 0x1ad1, 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c,
	0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1, 0xa33a, 0xb2b3,
	0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb,
	0xd68d, 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232,
	0x5ac5, 0x4b4c, 0x79d7, 0x685e, 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a,
	0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238, 0x93b1,
	0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9,
	0xf78f, 0xe606, 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330,
	0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3, 0x2c6a, 0x1ef1, 0x0f78
};

/* defines for transmit signalling */
enum {TOR_TXSIG_ONHOOK,TOR_TXSIG_OFFHOOK,TOR_TXSIG_START,TOR_TXSIG_KEWL} ;

/* states for transmit signalling */
typedef enum {TOR_TXSTATE_ONHOOK,TOR_TXSTATE_OFFHOOK,TOR_TXSTATE_START,
	TOR_TXSTATE_PREWINK,TOR_TXSTATE_WINK,TOR_TXSTATE_PREFLASH,
	TOR_TXSTATE_FLASH,TOR_TXSTATE_DEBOUNCE,TOR_TXSTATE_AFTERSTART,
	TOR_TXSTATE_RINGON,TOR_TXSTATE_RINGOFF,TOR_TXSTATE_KEWL} tor_txstate_t;

/* signalling bits */
#define	TOR_ABIT 8
#define	TOR_BBIT 4

#ifdef DEFAULT_TONE_ZONE
#undef CONFIG_TORMENTA_DEFAULT_TONE_ZONE
#define CONFIG_TORMENTA_DEFAULT_TONE_ZONE DEFAULT_TONE_ZONE
#endif

#define hdlc_to_tor(h) ((struct tor_softc *)(h))
#define dev_to_tor(d) ((struct tor_softc *)(d))

struct tor_tone {
	unsigned char *tonedata;	/* u-law tone data */
	int datalen;			/* Length of tone data */
	int tonesamples;		/* How long to play this tone before 
					   going to the next (in samples) */
	struct tor_tone *next;		/* Next tone in this sequence */
};

static struct tor_tone dtmf_tones[16];
static struct tor_tone mfv1_tones[15];

#include "digits.h"
	

struct tor_zone {
	char name[40];	/* Informational, only */
	int ringcadence[TOR_MAX_CADENCE];
	struct tor_tone *tones[TOR_TONE_MAX]; 
	/* Each of these is a circular list
	   of tor_tones to generate what we
	   want.  Use NULL if the tone is
	   unavailable */
};

#define DIGIT_MODE_DTMF 	0
#define DIGIT_MODE_MFV1	1

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
typedef struct wait_queue *wait_queue_head_t;
#endif

/* per-channel software structure */
static struct tor_softc {
#ifdef CONFIG_TORMENTA_NET
	/* HDLC stuff */
	hdlc_device 	netdev;	/* Net device -- MUST BE FIRST!!! */
#endif
	u_long		flags;	/* flags (you expected something else ??) */
	u_short 	*maddr;  /* io base address of card */
	int		irq;  /* irq */
/* the following are only meaningful to units other then the base (0) */
	wait_queue_head_t sel;	/* thingy for select stuff */
	int		blocksize;  /* transfer blocksize */
	int		sigtype;  /* signaling type */
	int		prewinktime;  /* pre-wink time (ms) */
	int		preflashtime;	/* pre-flash time (ms) */
	int		winktime;  /* wink time (ms) */
	int		flashtime;  /* flash time (ms) */
	int		starttime;  /* start time (ms) */
	int		rxwinktime;  /* rx wink time (ms) */
	int		rxflashtime; /* rx flash time (ms) */
	int		debouncetime;  /* FXS GS sig debounce time (ms) */
	
	/* Buffer declarations */
	u_char		*readbuf[MAX_NUM_BUFS];	/* read buffer */
	int		inreadbuf;
	int		outreadbuf;
	wait_queue_head_t readbufq; /* read wait queue */
	u_char		*writebuf[MAX_NUM_BUFS]; /* write buffers */
	int		inwritebuf;
	int		outwritebuf;
	wait_queue_head_t writebufq; /* write wait queue */
	u_char		eventbuf[TOR_MAX_EVENTSIZE];  /* event circ. buffer */
	wait_queue_head_t eventbufq; /* event wait queue */
	int		readn[MAX_NUM_BUFS];  /* # of bytes ready in read buf */
	int		readidx[MAX_NUM_BUFS];  /* current read pointer */
	int		writen[MAX_NUM_BUFS];  /* # of bytes ready in write buf */
	int		writeidx[MAX_NUM_BUFS];  /* current write pointer */
	
	int		numbufs;			/* How many buffers in channel */
	int		txbufpolicy;			/* Buffer policy */
	int		rxbufpolicy;			/* Buffer policy */
	int		txdisable;				/* Disable transmitter */
	int 	rxdisable;				/* Disable receiver */
	
	int		eventinidx;  /* out index in event buf (circular) */
	int		eventoutidx;  /* in index in event buf (circular) */
	u_int		itimer;	 /* timer for in signalling */
	u_int		otimer;  /* timer for out signalling */
	u_char		rxsig;  /* last rx signalling received */
	tor_txstate_t		txstate;  /* transmit state */
	wait_queue_head_t txstateq; /* tx state wait queue */
	int		iomask;  /* I/O Mux signal mask */
	short		rxlin; /* last received sample */
	short		txlin; /* last transmitted sample */
	short		conflast; /* last sample written to conf */
	short		conflast1;
	int		confn;	/* conference number */
	int		confmode;  /* conference mode */
	int		confmute; /* conference mute mode */
	int		gotgs; /* got ground-start flag (FXO GS) */
	struct tor_tone *curtone;		/* Current tone we're playing (if any) */
	struct tor_zone *curzone;		/* Zone for selecting tones */
	int		tonezone;				/* Zone as an integer */
	int    tonep;					/* Where we are in tone playing */

	int		digitmode;			/* What kind of tones are we sending? */
	char	txdialbuf[TOR_MAX_DTMF_BUF];
	int 	dialing;

	int		cadencepos;				/* Where in the cadence we are */

	int master;					/* Master channel */

	struct fasthdlc_state txhdlc;			/* HDLC state for transmitter */
	struct fasthdlc_state rxhdlc;			/* HDLC state for recevier */
	unsigned int	infcs;			/* Incoming FCS */
	unsigned int	outfcs;			/* Outgoing FCS */
#ifdef CONFIG_TORMENTA_ECHOCANCEL
	int echocancel;					/* Whether or not to perform echo cancellation */
	struct mark_ec ec;				/* Mark's echo canceller struct */
#endif
	
	spinlock_t	lock;				/* Lock for operations, instead of cli() */
} tor_softc[NTOR];

static int 		default_zone = CONFIG_TORMENTA_DEFAULT_TONE_ZONE;
static struct tor_zone *tone_zones[TOR_TONE_ZONE_MAX];
static spinlock_t 		zone_lock = SPIN_LOCK_UNLOCKED;

static u_char	txsigs[2][6];  /* transmit signal byte array save (copy of whats in chips) */

/* per-channel gain translation tables */
static u_char	rxgainx[NTOR][256]; /* receive gain translate table */
static u_char	txgainx[NTOR][256]; /* transmit gain translate table */

/* per-span span variables */
static u_int	alarms[2];  /* alarm state for spans */
static u_int	lastalarms[2];  /* last alarm state for spans */
static int	maints[2];  /* maint stat for spans */
static u_int	alarmtimer[2]; /* alarm timer */
static u_int	mainttimer[2]; /* maintance functions (loop codes) timer */
static wait_queue_head_t maintq[2]; /* maint wait queues */
static u_char	txlevels[2];  /* set tx level */
static u_char	lineconfig[2]; /* line configs */
static u_int	bpvcounts[2]; /* BPV counters */

/* sync sources. [0] is primary and [1] is secondary */
static u_char	syncs[2]; /* sync source info */

/* conference accumulators */
static short 	conf_sums[NCONF + 1];  /* current conference sums */
static short	conf_presums[NCONF + 1]; /* next conference sums */
static short	conf_prev[NCONF + 1];  /* last conference sums */

/* conference links */
static struct
{
	int	src;	/* source conf number */
	int	dst;	/* dst conf number */
} conf_links[NCONF + 1];

static u_char syncsrc;	/* active sync src */
/* the conversion tables get pre-loaded (for speed reasons) */
static short mulaw[256];  /* mulaw conversion table */
static u_char lin2mu[65536]; /* linear conversion table */

static int tor_major;  /* filled in with TOR major device number */

/* flags in softc */
#define	OPEN		1		/* channel is open */
#define	ALIVE		2		/* channel is alive */
#define	PSEUDO		4		/* is a pseudo-channel */
#define	STARTED		8		/* has been started */
#define CLEAR		16		/* treat as clearchannel */
#define HDLC		32		/* perform HDLC translation */
#define NETDEV		64		/* Network Device Allocated */
#define FCS		128			/* perform FCS calculation/verification */
#define NONBLOCKING	256
#define AUDIO		512		/* Treat as an audio channel as opposed to data */

#ifdef MODULE
static long base = 0;  /* physical memory base addr */
#else
static long base = CONFIG_TORMENTA_BASE;
#endif
static int irq = 0;  /* irq line */

#ifdef MODULE
/* get parameters from command line of insmod */
MODULE_PARM(base,"i"); 
MODULE_PARM(irq,"i");
#endif

/* macro-oni for determining a unit (channel) number */
#define	UNIT(file) MINOR(file->f_dentry->d_inode->i_rdev)

/* declarations for local functions (only used in this driver) */
static void tor_sethook(int chan, int what);
static void setctlreg(unsigned char val);
static unsigned char getctlreg(void);
static void t1out(int spanno,int loc,unsigned char val);
static unsigned char t1in(int spanno,int loc);
static unsigned char linear2ulaw(short sample);

/* declare interrupt handler */
static void torintr(int irq, void *dev_id, struct pt_regs *regs);

/* translations of data channels for 24 channels in a 32 bit PCM highway */
unsigned datxlt[] = { 0,
    1 ,2 ,3 ,5 ,6 ,7 ,9 ,10,11,13,14,15,17,18,19,21,22,23,25,26,27,29,30,31 };

/* This is the order that the data (audio) channels get
scanned in. This was done in this rather poopy manner because when outputting
(and inputting) a sine wave, such as in the case of TDD, any repeated samples
(because of PCM bus contention) will result in nasty-sounding distortion. The 
Mitel STPA chips (MT8920) have a contention mechanism, which results in a
situation where, if the processor accesses a timeslot that is currently
being transmitted or received, it will HOLD the bus until it is done with
the timeslot. This means that there can be cases where we are trying
to write to a timeslot, and its already outputting the same value
as the last one (since we didnt get there in time), and in a sine-wave
output, distortion will occur. In any other output, it will be utterly
un-noticeable. So, what we do is use a pattern that gives us the most
flexibility in how long our interrupt latency is (note: Even with this,
our interrupt latency must be between 4 and 28 microseconds!!!)  Essentially
we receive the interrupt just after the 24th channel is read.  It will
take us AT LEAST 30 microseconds to read it, but could take as much as
35 microseconds to read all the channels.  In any case it's the very
first thing we do in the interrupt handler.  Worst case (30 microseconds)
is that the MT8920 has only moved 7 channels.  That's where the 6 comes from.
*/

static int chseq[] = 
	{ 6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,1,2,3,4,5 } ;

  /* quiescent (idle) signalling states, for the various signalling types */
static u_char q_sig[8] = {0,0,TOR_BBIT,TOR_ABIT | TOR_BBIT,TOR_BBIT,
	TOR_BBIT,TOR_BBIT,TOR_BBIT} ;

/* clock values */
static u_char clockvals[] = {0,0x12,0x22,0};

/* names of tx level settings */
static char *txlevelnames[] = {
"0 db (CSU)/0-133 feet (DSX-1)",
"133-266 feet (DSX-1)",
"266-399 feet (DSX-1)",
"399-533 feet (DSX-1)",
"533-655 feet (DSX-1)",
"-7.5db (CSU)",
"-15db (CSU)",
"-22.5db (CSU)"
} ;

/* names of sync sources */
static char *syncsrcnames[] = {
"NONE (FREE-RUN)",
"SPAN 1",
"SPAN 2",
"NONE (FREE-RUN)"
} ;

/* set the control register */
static void setctlreg(val)
unsigned char val;
{
register char *cp;

	cp = (char *) &tor_softc[0].maddr[CTLREG];
	*cp = val;
}

/* get input from the status register */
static unsigned char getctlreg(void)
{
register char *cp;

	cp = (char *) &tor_softc[0].maddr[CTLREG];
	return(*cp);
}

/* output a byte to one of the registers in one of the Dallas T-1 chips */
static void t1out(spanno,loc,val)
int spanno;  /* span number (1,2) */
int loc;  /* address within the Dallas T-1 chip */
unsigned char val;  /* value to write there */
{
register int n;
register char *cp;

	  /* get the memory offset */
	n = spanno << 9;
	  /* point a char * at the address location */
	cp = (char *) &tor_softc[0].maddr[DADDR + n];
	*cp = loc;  /* set address in T1 chip */
	  /* point a char * at the data location */
	cp = (char *) &tor_softc[0].maddr[DDATA + n];
	*cp = val;  /* out the value */
}

/* get a byte from one of the registers in one of the Dallas T-1 chips */
static unsigned char t1in(spanno,loc)
int spanno;  /* span number (1,2) */
int loc;  /* address within the Dallas T-1 chip */
{
register int n;
register char *cp;

	  /* get the memory offset */
	n = spanno << 9;
	  /* point a char * at the address location */
	cp = (char *) &tor_softc[0].maddr[DADDR + n];
	*cp = loc;  /* set address in T1 chip */
	cp = (char *) &tor_softc[0].maddr[DDATA + n];
	  /* point a char * at the data location */
	return(*cp);
}

/* enqueue an event on a channel */
static void qevent(int chan, int event)
{
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[chan];

	  /* if full, ignore */
	if ((ss->eventoutidx == 0) && (ss->eventinidx == (TOR_MAX_EVENTSIZE - 1))) 
		return;
	  /* if full, ignore */
	if (ss->eventoutidx == (ss->eventinidx - 1)) return;
	  /* save the event */
	ss->eventbuf[ss->eventinidx++] = event;
	  /* wrap the index, if necessary */
	if (ss->eventinidx >= TOR_MAX_EVENTSIZE) ss->eventinidx = 0;
	  /* wake em all up */
	if (ss->iomask & TOR_IOMUX_SIGEVENT) wake_up_interruptible(&ss->eventbufq);
	wake_up_interruptible(&ss->readbufq);
	wake_up_interruptible(&ss->writebufq);
	wake_up_interruptible(&ss->sel);
	return;
}

/* do processing to determine if an alarm-associated event needs to be sent to
   associated channels */
static void alarm_notify(void)
{
int	i,j,k;

	  /* go thru both spans */
	for(i = 0; i < 2; i++)
	   {
		alarms[i] &= ~TOR_ALARM_LOOPBACK;
		/* determine current maint status */
		if (maints[i] || mainttimer[i]) alarms[i] |= TOR_ALARM_LOOPBACK;
		  /* if change in general state */
		if ((!alarms[i]) != (!lastalarms[i]))
		   {
			lastalarms[i] = alarms[i];  /* save state */
			  /* get associated event */
			if (!alarms[i])  /* if no more alarms */
				j = TOR_EVENT_NOALARM;
			else j = TOR_EVENT_ALARM;
			   /* send to all the chans */
			for(k = 1; k <= 24; k++)
			   {
				  /* if channel open, send event */
				/* if (tor_softc[(i * 24) + k].flags & OPEN) */
					qevent((i * 24) + k,j);
			   }
		   }
	   }
	return;
}

/* sleep in user space until woken up. Equivilant of tsleep() in BSD */
static int schluffen(wait_queue_head_t *q)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	struct wait_queue wait = { current, NULL };
#else
	DECLARE_WAITQUEUE(wait, current);
#endif
	add_wait_queue(q, &wait);
	current->state = TASK_INTERRUPTIBLE;
	if (!signal_pending(current)) schedule();
	current->state = TASK_RUNNING;
	remove_wait_queue(q, &wait);
	if (signal_pending(current)) return -ERESTARTSYS;
	return(0);
}

static int tone_zone_init(void)
{
	int x;
	static int inited = 0;
	spin_lock(&zone_lock);
	if (inited) {
		spin_unlock(&zone_lock);
		return 0;
	}
	for (x=0;x<TOR_TONE_ZONE_MAX;x++)
		tone_zones[x] = NULL;
	inited++;
	spin_unlock(&zone_lock);
	return 0;
}

static int free_tone_zone(int num)
{
	struct tor_zone *z;
	if ((num < 0) || (num >= TOR_TONE_ZONE_MAX))
		return -EINVAL;
	spin_lock(&zone_lock);
	z = tone_zones[num];
	tone_zones[num] = NULL;
	spin_unlock(&zone_lock);
	kfree(z);
	return 0;
}

int tor_register_tone_zone(int num, struct tor_zone *zone)
{
	int res=0;
	if ((num >= TOR_TONE_ZONE_MAX) || (num < 0))
		return -EINVAL;
	spin_lock(&zone_lock);
	if (tone_zones[num]) {
		res = -EINVAL;
	} else {
		res = 0;
		tone_zones[num] = zone;
	}
	spin_unlock(&zone_lock);
	if (!res)
		printk(KERN_INFO "Registered tone zone %d (%s)\n", num, zone->name);
	return res;
}

static int start_tone(struct tor_softc *ss, int tone)
{
	int res = -EINVAL;
	/* Stop the current tone, no matter what */
	ss->tonep = 0;
	ss->curtone = NULL;
	if ((tone >= TOR_TONE_MAX) || (tone < -1)) 
		return -EINVAL;
	/* Just wanted to stop the tone anyway */
	if (tone < 0)
		return 0;
	if (ss->curzone) {
		/* Have a tone zone */
		if (ss->curzone->tones[tone]) {
			ss->curtone = ss->curzone->tones[tone];
			res = 0;
		} else	/* Inidicate that zone is loaded but no such tone exists */
			res = -ENOSYS;
	} else	/* Note that no tone zone exists at the moment */
		res = -ENODATA;
	return res;
}

static int set_tone_zone(struct tor_softc *ss, int zone)
{
	int res=0;
	/* Assumes channel is already locked */
	if ((zone >= TOR_TONE_ZONE_MAX) || (zone < -1))
		return -EINVAL;
	
    spin_lock(&zone_lock);
	if (zone == -1) {
		zone = default_zone;
	}
	if (tone_zones[zone]) {
		ss->curzone = tone_zones[zone];
		ss->tonezone = zone;
	} else {
		res = -ENODATA;
	}
	
	spin_unlock(&zone_lock);
	return res;
}

/* device probe routine .. determines if the Tormenta device is present in
   the system */
static int
tor_probe(void)
{
int			i,status;
u_char			c1,c2;
struct	tor_softc	*ss=(struct tor_softc *)&tor_softc[0];

	status = 0; /* default status return is 'not present' */

	ss->flags = 0;  /* zero flags */

	  /* initialize control register */
	setctlreg(MASTERCLOCK);

	   /* init all the registers in first T-1 chip to 0 */
	for(i = 0x20; i < 0x40; i++) t1out(1,i,0); /* set register to 0 */
	for(i = 0x60; i < 0x80; i++) t1out(1,i,0); /* set register to 0 */
	/* simple test that will fail if tried in an array of standard memory */
	  /* put an 0x55 here */
	t1out(1,0x2b,0x55);
	  /* put an 0xaa here */
	t1out(1,0x2c,0xaa);
	  /* get input from first location */
	c1 = t1in(1,0x2b);
	  /* get input from second location */
	c2 = t1in(1,0x2c);
	  /* see if we read back what we put in */
	if ((c1 == 0x55) && (c2 == 0xaa))
	   {
		/* Try to get the irq if the user didn't specify one */
		if (!irq) {
			autoirq_setup(0);
			setctlreg(MASTERCLOCK|INTENA);
			/* Wait a jiffie -- that's plenty of time */
			irq = autoirq_report(5);
		}
		if (irq == 2)
			irq = 9;
		if (irq) {
			status = 1;	/* found */
			ss->flags |= ALIVE;
			printk("Tormenta Card found at base addr 0x%lx, irq %d\n",base,irq);
		} else
			printk("Tormenta Card found at base addr 0x%lx, but unable to determine IRQ.  Try using irq= option\n", base);
	   }
	return(status);
}

static void set_clear(void)
{
	int i,j;
	unsigned short val=0;
	
	for (i=0;i<NTOR-1;i++) {
		j = (i/8) % 3;
		if (tor_softc[i+1].flags & PSEUDO)
			break;

		if (tor_softc[i+1].flags & CLEAR) 
			val |= 1 << (i % 8);

		if ((i % 8)==7) {
#if 0
			printk("Putting %d in register %02x on span %d\n",
			       val, 0x39 + j, 1 + i/24);
#endif
			t1out(1 + i/24, 0x39 + j, val);
			val = 0;
		}
	}
		
}

static void init_channel(int);

static void tor_shutdown(void)
{
	long flags;
	int i,j;
#ifdef CONFIG_TORMENTA_NET
	int x;
#endif	
	/* Shut down the card, free interfaces, etc */
	  /* initialize */
	spin_lock_irqsave(&tor_softc[0].lock, flags);
	setctlreg(MASTERCLOCK);

	  /* if started, get ridda irq */
	if (tor_softc[0].flags & STARTED) {
		free_irq(irq,NULL);
		/* init both T1 chips */
		for(j = 1; j <= 2; j++)
		{
			printk("Shutting down span %d: ",j);
			/* init all the registers to 0 */
			for(i = 0x20; i < 0x40; i++) t1out(j,i,0); /* set register to 0 */
			for(i = 0x60; i < 0x80; i++) t1out(j,i,0); /* set register to 0 */
			printk("Done\n");
		}
	}

#ifdef CONFIG_TORMENTA_NET
	  /* Unregister any interfaces as appropriate */
	for (x=1;x<NTOR;x++) 
		if (tor_softc[x].flags & NETDEV) {
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
			printk("Unregistering device '%s'\n", tor_softc[x].netdev.name);
#else
			printk("Unregistering device '%s'\n", tor_softc[x].netdev.netdev.name);
#endif
			unregister_hdlc_device(&tor_softc[x].netdev);
			tor_softc[x].flags &= ~NETDEV;
		}
#endif		
	  /* Free any buffers */
	for (i=1;i < NTOR; i++) {
		if (tor_softc[i].readbuf[0]);
			kfree(tor_softc[i].readbuf[0]);
	}
	for(i = 1; i < NTOR; i++) 
		init_channel(i);

	tor_softc[0].flags &= ~STARTED;
	setctlreg(0);

	spin_unlock_irqrestore(&tor_softc[0].lock, flags);
	
}

/* set line parameters and start system */
static void tor_startup(struct tor_lineconfig *lc)
{
	int	i,j;
	unsigned int endjif;
	unsigned char val;
	int flags;
	struct	tor_softc	*ss= &tor_softc[0];

	spin_lock_irqsave(&tor_softc[0].lock, flags);
		  /* initialize all of the system variables */
	memset(txsigs,0,sizeof(txsigs));
	memset(alarms,0,sizeof(alarms));
	memset(lastalarms,0,sizeof(lastalarms));
	memset(maints,0,sizeof(maints));
	memset(txlevels,0,sizeof(txlevels));
	memset(alarmtimer,0,sizeof(alarmtimer));
	memset(mainttimer,0,sizeof(mainttimer));
	memset(bpvcounts,0,sizeof(bpvcounts));
	memset(syncs,0,sizeof(syncs));
	memset(conf_sums,0,sizeof(conf_sums));
	memset(conf_presums,0,sizeof(conf_presums));
	memset(conf_prev,0,sizeof(conf_prev));
	memset(conf_links,0,sizeof(conf_links));

	  /* initialize control register */
	setctlreg(MASTERCLOCK);

	  /* get the tx level information from the flags */
	txlevels[0] = lc->lbo[0];
	txlevels[1] = lc->lbo[1];

	  /* save line configs */
	lineconfig[0] = lc->lineconfig[0];
	lineconfig[1] = lc->lineconfig[1];

	  /* get the sync source information from the flags */
	syncs[0] = lc->psync;
	syncs[1] = lc->ssync;


	  /* enable 16 bit memory cycles (so we can access STPA chips) */
	setctlreg(MASTERCLOCK | ENA16);

	  /* init both STPA's to all silence */
	for(i = 0; i < 32; i++) tor_softc[0].maddr[i] = 0x7f7f;

	  /* disable 16 bit memory cycles */
	setctlreg(MASTERCLOCK);

	  /* init both T1 chips */
	for(j = 1; j <= 2; j++)
	   {

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	   	init_waitqueue_head(&maintq[j-1]);
#endif
		printk("Span %d: ",j);
		   /* init all the registers to 0 */
		for(i = 0x20; i < 0x40; i++) t1out(j,i,0); /* set register to 0 */
		for(i = 0x60; i < 0x80; i++) t1out(j,i,0); /* set register to 0 */

		t1out(j,0x2b,8); /* set RCR1 to 8, full-on sync required */
		t1out(j,0x2c,8); /* set RCR2 to 8, RSYNC is an input */
		t1out(j,0x35,0x10); /* set TCR1 to 0x10 (RBS enable) */
		   /* note also set bit 0 in TCR1 to send yellow alarm */
		t1out(j,0x36,4); /* set TCR2 to 4, TSYNC to be output */
		t1out(j,0x37,0x8c); /* set CCR1 to 0x8c, Tx & Rx elastic store and
		   sysclk = 2.048 mhz. Loopback controls are here also */
		 /* set CCR2 to 0x20, Enable F bits pattern  */
		i = 0x20;
		  /* if ESF, set accordingly */
		if (lineconfig[j - 1] & TOR_CONFIG_ESF)
			i = 0x88;
		   /* if B8ZS, add that, too */
		if (lineconfig[j - 1] & TOR_CONFIG_B8ZS)
			i |= 0x44;
		t1out(j,0x38,i);  /* output to CCR2 */
		  /* print line coding type */
		if (i & 0x40)
		   printk("B8ZS ");
		else
		   printk("AMI ");
		   /* print framing type */
		if (i & 0x80)
		   {
		   printk("ESF ");
		   }
		else
		   {
		   printk("D4 (SF) ");
		   t1out(j,0x7e,0x1c); /* F bits pattern (0x1c) into FDL register */
		   }
		 /* set LICR to 0 plus tx build-out  */
		t1out(j,0x7c,txlevels[j - 1] << 5);
		printk(", Line build-out: %s\n",
			txlevelnames[txlevels[j - 1]]);
		if (lineconfig[j - 1] & TOR_CONFIG_NOTOPEN)
			printk("      Will assert Yellow Alarm if no chans. open\n");
		t1out(j,0x30,1);  /* set LIRST to 1 in CCR3 */
		endjif = jiffies + 10;
		spin_unlock_irqrestore(&tor_softc[0].lock, flags);
		while(jiffies < endjif); /* wait 100 ms */
		spin_lock_irqsave(&tor_softc[0].lock, flags);
		t1out(j,0x30,0x40); /* set CCR3 to 0x40, resetting Elastic Store */
	   }
	printk("Primary Sync Source: %s\n",syncsrcnames[syncs[0]]);
	printk("Secondary Sync Source: %s\n",syncsrcnames[syncs[1]]);
	  /* enable interrupts and set clock value */
	syncsrc = 0;  /* start with free-run */
	if (syncs[0]) syncsrc = syncs[0]; /* if we have primary, use it */
	else if (syncs[1]) syncsrc = syncs[1]; /* otherwise, if we
		have secondary, use it */
	  /* enable transparency on clear channels */
	val = 0;
	  /* output the clock info and enable interrupts */
	setctlreg(clockvals[syncsrc] | INTENA);
	/* 
	 *  Set up mu-law conversion table
	 */
	for(i = 0;i < 256;i++)
	   {
		short mu,e,f,y;
		static short etab[]={0,132,396,924,1980,4092,8316,16764};

		mu = 255-i;
		e = (mu & 0x70)/16;
		f = mu & 0x0f;
		y = f * (1 << (e + 3));
		y += etab[e];
		if (mu & 0x80) y = -y;
	        mulaw[i] = y;
	   }
	  /* set up the reverse (mu-law) conversion table */
	for(i = 0; i < 65536; i++)
	   {
		lin2mu[i] = linear2ulaw(i - 32768);
	   }

	if 	(!(tor_softc[0].flags & STARTED)) {
		/* Only request the IRQ if we haven't been started yet */
		if (request_irq(irq,torintr,SA_INTERRUPT,"tor",NULL))
		   {
			printk("IRQ %d cannot be used for some reason!!\n",irq);
		   }

	}
	tor_softc[0].flags |= STARTED;
	  /* setup clear channels appropriately */
	set_clear();
	for(i = 1; i < NTOR; i++) 
	   {
		if (!(tor_softc[i].flags & CLEAR)) {
			/* put on hook */
			tor_sethook(i,TOR_TXSIG_ONHOOK);
			/* initialize to quiescent signalling value (what one would
			   expect to see from an idle trunk) */
			ss->rxsig = q_sig[tor_softc[i].sigtype];
		}
	   }
	spin_unlock_irqrestore(&tor_softc[0].lock, flags);
}

static int tor_reallocbufs(struct tor_softc *, int bufsize, int numbufs);

#ifdef CONFIG_TORMENTA_NET

static int tor_net_open(hdlc_device *hdlc)
{
	struct tor_softc *ms = hdlc_to_tor(hdlc);
	int res;
	if (!ms) {
		printk("tor_net_open: nothing??\n");
		return -EINVAL;
	}
	if (ms->flags & OPEN) {
		printk("%d is already open!\n", ms->master);
		return -EBUSY;
	}
	if (!(ms->flags & NETDEV)) {
		printk("%d is not a net device!\n", ms->master);
		return -EINVAL;
	}
	ms->txbufpolicy = POLICY_IMMEDIATE;
	ms->rxbufpolicy = POLICY_IMMEDIATE;

	res = tor_reallocbufs(ms, TOR_DEFAULT_MTU_MRU, DEFAULT_NUM_BUFS);
	if (res) 
		return res;

	fasthdlc_init(&ms->rxhdlc);
	fasthdlc_init(&ms->txhdlc);
	ms->infcs = PPP_INITFCS;
	ms->outfcs = PPP_INITFCS;

	MOD_INC_USE_COUNT;
#if 0
	printk("TORNET: Opened channel %d\n", ms->master);
#endif
	return 0;
}

static void tor_net_close(hdlc_device *hdlc)
{
	struct tor_softc *ms = hdlc_to_tor(hdlc);
	if (!ms) {
		printk("tor_net_close: nothing??\n");
		return;
	}
	if (!(ms->flags & NETDEV)) {
		printk("%d is not a net device!\n", ms->master);
		return;
	}
	/* Not much to do here.  Just deallocate the buffers */
	tor_reallocbufs(ms, 0, 0);
	MOD_DEC_USE_COUNT;
	return;
}

#endif

static inline void calc_fcs(struct tor_softc *ss)
{
	int x;
	unsigned int fcs=PPP_INITFCS;
	unsigned char *data = ss->writebuf[ss->inwritebuf];
	int len = ss->writen[ss->inwritebuf];
	/* Not enough space to do FCS calculation */
	if (len < 2)
		return;
	for (x=0;x<len-2;x++)
		fcs = PPP_FCS(fcs, data[x]);
	fcs ^= 0xffff;
	/* Send out the FCS */
	data[len-2] = (fcs & 0xff);
	data[len-1] = (fcs >> 8) & 0xff;
}

#ifdef CONFIG_TORMENTA_NET
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
static int tor_xmit(struct sk_buff *skb, struct device *dev)
#else
static int tor_xmit(hdlc_device *hdlc, struct sk_buff *skb)
#endif
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	struct tor_softc *ss = dev_to_tor(dev);
#else
	struct tor_softc *ss = hdlc_to_tor(hdlc);
	struct net_device *dev = &ss->netdev.netdev;
#endif	
	int retval = 1;
	int x,oldbuf;
	unsigned int fcs;
	unsigned char *data;
	long flags;
	/* See if we have any buffers */
	spin_lock_irqsave(&ss->lock, flags);
	if (skb->len > ss->blocksize - 2) {
		printk(KERN_ERR "tor_xmit(%s): skb is too large (%d > %d)\n", dev->name, skb->len, ss->blocksize -2);
		ss->netdev.stats.tx_dropped++;
		retval = 0;
	} else if (ss->inwritebuf >= 0) {
		/* We have a place to put this packet */
		/* XXX We should keep the SKB and avoid the memcpy XXX */
		data = ss->writebuf[ss->inwritebuf];
		memcpy(data, skb->data, skb->len);
		ss->writen[ss->inwritebuf] = skb->len;
		ss->writeidx[ss->inwritebuf] = 0;
		/* Calculate the FCS */
		fcs = PPP_INITFCS;
		for (x=0;x<skb->len;x++)
			fcs = PPP_FCS(fcs, data[x]);
		/* Invert it */
		fcs ^= 0xffff;
		/* Send it out LSB first */
		data[ss->writen[ss->inwritebuf]++] = (fcs & 0xff);
		data[ss->writen[ss->inwritebuf]++] = (fcs >> 8) & 0xff;
#if 0
		do_fcs(data, ss->writen[ss->inwritebuf], "transmitter");
#endif
		/* Advance to next window */
		oldbuf = ss->inwritebuf;
		ss->inwritebuf = (ss->inwritebuf + 1) % ss->numbufs;

		if (ss->inwritebuf == ss->outwritebuf) {
			/* Whoops, no more space.  */
			ss->inwritebuf = -1;
		}
		if (ss->outwritebuf < 0) {
			/* Let the interrupt handler know there's
			   some space for us */
			ss->outwritebuf = oldbuf;
		}
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
		dev->trans_start = jiffies;
#endif
#if 0
		printk("Buffered %d bytes to go out in buffer %d\n", ss->writen[oldbuf], oldbuf);
		for (x=0;x<ss->writen[oldbuf];x++)
		     printk("%02x ", ss->writebuf[oldbuf][x]);
		printk("\n");
#endif
		retval = 0;
		/* Free the SKB */
		dev_kfree_skb(skb);
	}
	spin_unlock_irqrestore(&ss->lock, flags);
	return retval;
}

static int tor_net_ioctl(hdlc_device *hdlc, struct ifreq *ifr, int cmd)
{
	return -EIO;
}

#endif

static int tor_reallocbufs(struct tor_softc *ss, int j, int numbufs)
{
	unsigned char *newbuf, *oldbuf;
	long flags;
	int x;
	/* Check numbufs */
	if (numbufs < 2)
		numbufs = 2;
	if (numbufs > MAX_NUM_BUFS)
		numbufs = MAX_NUM_BUFS;
	/* We need to allocate our buffers now */
	if (j) {
		newbuf = kmalloc(j * 2 * numbufs, GFP_KERNEL);
		if (!newbuf) 
			return (-ENOMEM);
	} else
		newbuf = NULL;
	  /* Now that we've allocated our new buffer, we can safely
	     move things around... */
	spin_lock_irqsave(&ss->lock, flags);
	ss->blocksize = j; /* set the blocksize */
	oldbuf = ss->readbuf[0]; /* Keep track of the old buffer */
	
	if (newbuf) {
		for (x=0;x<numbufs;x++) {
			ss->readbuf[x] = newbuf + x * j;
			ss->writebuf[x] = newbuf + (numbufs + x) * j;
		}
	} else {
		for (x=0;x<numbufs;x++) {
			ss->readbuf[x] = NULL;
			ss->writebuf[x] = NULL;
		}
	}
	/* Mark all buffers as empty */
	for (x=0;x<numbufs;x++) 
		ss->writen[x] = 
		ss->writeidx[x]=
		ss->readn[x]=
		ss->readidx[x] = 0;
	
	/* Keep track of where our data goes (if it goes
	   anywhere at all) */
	if (newbuf) {
		ss->inreadbuf = 0;
		ss->inwritebuf = 0;
	} else {
		ss->inreadbuf = -1;
		ss->inwritebuf = -1;
	}
	ss->outreadbuf = -1;
	ss->outwritebuf = -1;
	ss->numbufs = numbufs;
	if (ss->txbufpolicy == POLICY_WHEN_FULL)
		ss->txdisable = 1;
	else
		ss->txdisable = 0;

	if (ss->rxbufpolicy == POLICY_WHEN_FULL)
		ss->rxdisable = 1;
	else
		ss->rxdisable = 0;

	spin_unlock_irqrestore(&ss->lock, flags);
	if (oldbuf)
		kfree(oldbuf);
	return 0;
}

/* Set line configuration on a channel (and initialize it) */
static int tor_chanconfig(int unit,int flags, int master)
{
	int	i;
	struct	tor_softc	*ss= &tor_softc[unit];
#ifdef CONFIG_TORMENTA_NET
	int	res;
#endif
	int	x;
	int oldsig;

	oldsig = ss->sigtype;
	  /* get signalling type from flags */
	ss->sigtype = flags & 0xff;
	  /* for non-master units, just copy the IO base */
	ss->maddr = tor_softc[0].maddr;
	  /* and IRQ of course */
	ss->irq = tor_softc[0].irq;

#ifdef CONFIG_TORMENTA_NET
	  /* Unregister our network device */
	if (ss->flags & NETDEV)  {
		/* Lets make the user ifconfig it down before
		   we unregister the device */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
		if (ss->netdev.dev.flags & IFF_UP) {
#else
		if (ss->netdev.netdev.flags & IFF_UP) {
#endif
			printk(KERN_WARNING "Can't switch out of HDLC net mode on unit %d, since the current interface is up\n", unit);
			return -EBUSY;
		}
		unregister_hdlc_device(&ss->netdev);
		ss->flags &= ~NETDEV;
	}
	  /* If appropriate register our network device before we continue */
	if (ss->sigtype == TOR_HDLCNET) {
		printk("Trying to register a TORNET device\n");
		  /* if we're a nethdlc, register ourselves */
		memset(&ss->netdev, 0, sizeof(ss->netdev));
		
		/* Setup HDLC routines */
		ss->netdev.ioctl = tor_net_ioctl;
		ss->netdev.open = tor_net_open;
		ss->netdev.close = tor_net_close;
		ss->netdev.set_mode = NULL;
		
		/* Setup netdev routines */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
		ss->netdev.dev.hard_start_xmit = tor_xmit;
		ss->netdev.dev.irq=ss->irq;
		ss->netdev.dev.tx_queue_len=50;
#else
		ss->netdev.xmit = tor_xmit;
		ss->netdev.netdev.irq=ss->irq;
		ss->netdev.netdev.tx_queue_len=50;
#endif		
		res = register_hdlc_device(&ss->netdev);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
		printk("Register HDLC device: %s (%d)\n", (ss->netdev.name ? ss->netdev.name : "(unknown)"), res);
		if (res) {
			printk("Failed to register HDLC device for channel %d (%d)\n", ss->master, res);
			return res;
		} else
#else
		printk("Register HDLC device: %s (%d)\n", (ss->netdev.netdev.name ? ss->netdev.netdev.name : "(unknown)"), res);
		if (res) {
			printk("Failed to register HDLC device for channel %d (%d)\n", ss->master, res);
			return res;
		} else
#endif
			ss->flags |= NETDEV;
	}
#endif
	  /* intitialize hdlc state */
	fasthdlc_init(&ss->rxhdlc);
	fasthdlc_init(&ss->txhdlc);
	ss->outfcs = PPP_INITFCS;
	ss->infcs = PPP_INITFCS;
	  /* if clear channel, note it... */
	if ((ss->sigtype == TOR_CLEAR) ||
	    (ss->sigtype == TOR_HDLCRAW) ||
	    (ss->sigtype == TOR_HDLCNET) ||
	    (ss->sigtype == TOR_HDLCFCS) ||
	    (ss->sigtype == TOR_SLAVE)) {
		ss->flags |= CLEAR;
		ss->flags &= ~AUDIO;
	} else {
		ss->flags &= ~CLEAR;
		ss->flags |= AUDIO;
	}
	  /* if hdlc encapsulation, note it... */
	if ((ss->sigtype == TOR_HDLCRAW) ||
	    (ss->sigtype == TOR_HDLCNET) ||
	    (ss->sigtype == TOR_HDLCFCS))
	    	ss->flags |= HDLC;
	else
		ss->flags &= ~HDLC;
	  /* if hdlc encapsulation, note it... */
	if ((ss->sigtype == TOR_HDLCFCS) ||
	    (ss->sigtype == TOR_HDLCNET))
	    	ss->flags |= FCS;
	else
		ss->flags &= ~FCS;
		
	if (ss->sigtype == TOR_SLAVE)
		ss->master = master;
	else
		ss->master = unit;

	  /* if a pseudo-type (from flags), set PSEUDO flag */
	if (flags & TOR_PSEUDO) 
		ss->flags |= PSEUDO;
	else
		ss->flags &= ~PSEUDO;
	
	if (!(ss->flags & CLEAR)) {
		/* If we're not clear, be sure nobody has us
		   as a master */
		for (x=1;x<NTOR;x++)
			if (tor_softc[x].master == unit)
				tor_softc[x].master = x;
	}
	ss->flags |= ALIVE;
	  /* initialize timers */
	ss->itimer = ss->otimer = 0;
	ss->txstate = TOR_TXSTATE_ONHOOK;
	  /* initialize IO MUX mask */
	ss->iomask = 0;
	   /* initialize conference variables */
	ss->confn = 0;
	ss->confmode = 0;
	ss->conflast = 0;
	ss->conflast1 = 0;
	ss->confmute = 0;
	ss->gotgs = 0;
	  /* init the translation tables */
	for(i = 0; i < 256; i++) txgainx[unit][i] = 
		rxgainx[unit][i] = i;
	if (tor_softc[0].flags & STARTED) {
		/* If the span is already started, we need to hang this one
		   up, and also setup the CLEARness */
		if (!(ss->flags & CLEAR)) {
			tor_sethook(unit, TOR_TXSIG_ONHOOK);
			ss->rxsig = q_sig[ss->sigtype];  
		}
#if 0
		printk("Trying to do chanconfig on already started channel...\n");
#endif
		set_clear();
	}
	return 0;
}

/* device open routine */
static	int
tor_open(struct inode *inode, struct file *file)
{
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[UNIT(file)];
int	i,res;
long flags;

	  /* if device not present, cant do I/O */
	if((ss->flags & ALIVE) == 0)
		return -ENXIO;

	  /* if not master and already open, cant do it again */
	if(UNIT(file) && (ss->flags & OPEN))
		return -EBUSY;
	  /* make sure we're not part of a net dev */
	if(UNIT(file) && (ss->flags & NETDEV))
		return -EBUSY;
	  /* make sure we're not a slave channel */
	if(UNIT(file) && (UNIT(file) != ss->master))
		return -EBUSY;
		
	ss->rxbufpolicy = POLICY_IMMEDIATE;
	ss->txbufpolicy = POLICY_IMMEDIATE;

	ss->txdisable = 0;
	ss->rxdisable = 0;

	ss->digitmode = DIGIT_MODE_DTMF;
	ss->dialing = 0;

	ss->cadencepos = 0;

	/* Allocate buffers */
	if ((res = tor_reallocbufs(ss, TOR_DEFAULT_BLOCKSIZE, DEFAULT_NUM_BUFS)))
		return res;

	/* Initialize HDLC state and FCS's */
	fasthdlc_init(&ss->rxhdlc);
	fasthdlc_init(&ss->txhdlc);
	ss->infcs = PPP_INITFCS;
	ss->outfcs = PPP_INITFCS;
		
	spin_lock_irqsave(&ss->lock, flags);
	ss->flags |= OPEN;  /* set open flag */
	  /* set up default timing parameters */   
	ss->prewinktime = TOR_DEFAULT_PREWINKTIME;
	ss->preflashtime = TOR_DEFAULT_PREFLASHTIME;
	ss->winktime = TOR_DEFAULT_WINKTIME;
	ss->flashtime = TOR_DEFAULT_FLASHTIME;
	if ((ss->sigtype == TOR_FXOLS) || (ss->sigtype == TOR_FXOGS))
		ss->starttime = TOR_DEFAULT_RINGTIME;
	else ss->starttime = TOR_DEFAULT_STARTTIME;
	ss->rxwinktime = TOR_DEFAULT_RXWINKTIME;
	ss->rxflashtime = TOR_DEFAULT_RXFLASHTIME;
	ss->debouncetime = TOR_DEFAULT_DEBOUNCETIME;
	  /* initialize timers */
	ss->itimer = ss->otimer = 0;
#ifdef CONFIG_TORMENTA_ECHOCANCEL
	  /* initialize echo canceller if appropriate */
	ss->echocancel = 0;  
#endif	
	  /* initialize transmit signlling state */
	ss->txstate = TOR_TXSTATE_ONHOOK;
	  /* start with channel on hook */
	tor_sethook(UNIT(file),TOR_TXSIG_ONHOOK);
	  /* initialize the 'select' structure */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	ss->sel = 0;
	ss->readbufq = 0;
	ss->writebufq = 0;
	ss->eventbufq = 0;
	ss->txstateq = 0;
#else
	init_waitqueue_head(&ss->sel);
	init_waitqueue_head(&ss->readbufq);
	init_waitqueue_head(&ss->writebufq);
	init_waitqueue_head(&ss->eventbufq);
	init_waitqueue_head(&ss->txstateq);
#endif
	
	  /* initialize the IO MUX mask */
	ss->iomask = 0;
	  /* initialize the conference variables */
	ss->confn = 0;
	ss->confmode = 0;
	ss->conflast = 0;
	ss->conflast1 = 0;
	ss->confmute = 0;
	ss->gotgs = 0;
	if (UNIT(file)) {
		ss->curtone = NULL;
		ss->tonep = 0;
		set_tone_zone(ss, -1);
	}
	  /* init the translation tables */
	for(i = 0; i < 256; i++) txgainx[UNIT(file)][i] = 
		rxgainx[UNIT(file)][i] = i;
	  /* Clear any pending events */
	ss->eventinidx = ss->eventoutidx = 0;
	spin_unlock_irqrestore(&ss->lock, flags);
	MOD_INC_USE_COUNT;
	return 0;
}

/* device close routine */
static	int
tor_release(struct inode *inode, struct file *file)
{
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[UNIT(file)];
int	i;
long flags;
	/* Get rid of buffers (can't fail) */
	tor_reallocbufs(ss, 0, 0);
	spin_lock_irqsave(&ss->lock, flags);
	ss->curtone = NULL;
	ss->curzone = NULL;
	ss->cadencepos = 0;
	ss->flags &= ~OPEN; /* clear channel open flag */
	  /* force channel (and signalling state) on hook */
	if (ss->sigtype == TOR_FXOKS)
	   {
		if (ss->txstate != TOR_TXSTATE_KEWL)
		   {
			tor_sethook(UNIT(file),TOR_TXSIG_KEWL);
			ss->otimer = TOR_KEWLTIME;
			ss->txstate = TOR_TXSTATE_KEWL;
		   }
	   }
	else
	   {
		tor_sethook(UNIT(file),TOR_TXSIG_ONHOOK);
		ss->otimer = 0;
		ss->txstate = TOR_TXSTATE_ONHOOK;
	   }
	  /* initialize channel timers */
	ss->itimer = 0;
	  /* initialize 'select' structure */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	ss->sel = 0;
	ss->readbufq = 0;
	ss->writebufq = 0;
	ss->eventbufq = 0;
	ss->txstateq = 0;
#else
	init_waitqueue_head(&ss->sel);
	init_waitqueue_head(&ss->readbufq);
	init_waitqueue_head(&ss->writebufq);
	init_waitqueue_head(&ss->eventbufq);
	init_waitqueue_head(&ss->txstateq);
#endif
	/* De-queue any outgoing DTMF */
	ss->txdialbuf[0] = '0';
	ss->digitmode = DIGIT_MODE_DTMF;
	ss->dialing = 0;
	
#ifdef CONFIG_TORMENTA_ECHOCANCEL
	  /* initialize echo canceller if appropriate */
	ss->echocancel = 0;  
#endif	

	  /* initialize IO MUX mask */
	ss->iomask = 0;
	  /* initialize conference variables */
	ss->confn = 0;
	ss->confmode = 0;
	ss->conflast = 0;
	ss->conflast1 = 0;
	ss->confmute = 0;
	ss->gotgs = 0;
	  /* init the translation tables */
	for(i = 0; i < 256; i++) txgainx[UNIT(file)][i] = 
		rxgainx[UNIT(file)][i] = i;
	spin_unlock_irqrestore(&ss->lock, flags);
	MOD_DEC_USE_COUNT;
	return 0;
}

/* device write routine */
static	ssize_t
tor_write(struct file *file, const char *usrbuf, size_t count, loff_t *ppos)
{
int	amnt,rv,oldbuf,res;
long flags;
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[UNIT(file)];

	   /* fail if invalid byte count */
	if (count < 1) return(-EINVAL);
#if 0
	printk("pre: %d buffers, transmitter at %d, us at %d (disabled = %d)\n", ss->numbufs, ss->outwritebuf, ss->inwritebuf, ss->txdisable);
#endif
	/* loop until we have a buffer to use */
	for(;;)   {
		spin_lock_irqsave(&ss->lock, flags);
		if (ss->curtone) {
			/* Turn off any current tone that is playing. Stop any dial buffer */
			ss->curtone = NULL;
			ss->tonep = 0;
			ss->txdialbuf[0] = '0';
		}
		  /* return if event pending */
		if (ss->eventinidx != ss->eventoutidx)
		   {
			spin_unlock_irqrestore(&ss->lock, flags);
			return(-ELAST);
		   }
		res = ss->inwritebuf;
		spin_unlock_irqrestore(&ss->lock, flags);
		if (res >= 0) break;  /* if found one */
		  /* sleep until we have one to use */
		if (file->f_flags & O_NONBLOCK)
			return -EAGAIN;
		rv = schluffen(&ss->writebufq);
		if (rv) return(rv);
	   }
	  /* figure out how many to actually transfer */
	amnt = count;
	  /* limit to max blocksize */
	if (amnt > ss->blocksize) amnt = ss->blocksize;
	if (amnt) /* if there is any */
	   {
		copy_from_user(ss->writebuf[ss->inwritebuf],usrbuf,amnt);
		ss->writen[ss->inwritebuf] = amnt;  /* set value, activating it */
		ss->writeidx[ss->inwritebuf] = 0;
		if (ss->flags & FCS)
			calc_fcs(ss);
		oldbuf = ss->inwritebuf;
		spin_lock_irqsave(&ss->lock, flags);
		ss->inwritebuf = (ss->inwritebuf + 1 ) % ss->numbufs;
		if (ss->inwritebuf == ss->outwritebuf) {
			/* Whoops, lets not step on the receiver, but instead
			   wait for them to wake us up. */
			ss->inwritebuf = -1;
			/* Enable the transmitter in case they're on POLICY_WHEN_FULL */
			ss->txdisable = 0;
		}
		if (ss->outwritebuf < 0) {
			/* Okay, the interrupt handler has been WAITING for us
			   to finally have some data.  Give them a buffer
			   they can use. */
			ss->outwritebuf = oldbuf;
		}
#if 0
		printk("post: %d buffers, transmitter at %d, us at %d (disabled = %d)\n", ss->numbufs, ss->outwritebuf, ss->inwritebuf, ss->txdisable);
#endif		
		spin_unlock_irqrestore(&ss->lock, flags);
	   }
	return(amnt);
}

/* device read routine */
static	ssize_t
tor_read(struct file *file, char *usrbuf, size_t count, loff_t *ppos)
{
int	amnt,oldbuf,rv,res;
long flags;
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[UNIT(file)];

	   /* fail if invalid byte count */
	if (count < 1) return(-EINVAL);

	/* loop until we have a buffer to use */
	for(;;)
	   {
		spin_lock_irqsave(&ss->lock, flags);
		  /* return if event pending */
		if (ss->eventinidx != ss->eventoutidx)
		   {
			spin_unlock_irqrestore(&ss->lock, flags);
			return(-ELAST);
		   }
		res = ss->outreadbuf;
		/* Force to -1 if we have receive disabled */
		if (ss->rxdisable)
			res = -1;
		spin_unlock_irqrestore(&ss->lock, flags);
		if (res >= 0) break;  /* if found one */
		  /* sleep until we have one to use unless we
		     are non-blocking */
		if (file->f_flags & O_NONBLOCK)
			return -EAGAIN;
		rv = schluffen(&ss->readbufq);
		if (rv) return(rv);
	   }
	  /* figure out how many to actually transfer */
	amnt = count;
	  /* limit to how many in buffer */
	if (amnt > ss->readn[ss->outreadbuf]) amnt = ss->readn[ss->outreadbuf];
	if (amnt) copy_to_user(usrbuf,ss->readbuf[ss->outreadbuf],amnt);
	spin_lock_irqsave(&ss->lock, flags);
	ss->readidx[ss->outreadbuf] = 0;
	ss->readn[ss->outreadbuf] = 0; /* de-allocate it */
	oldbuf = ss->outreadbuf;
	ss->outreadbuf = (ss->outreadbuf + 1) % ss->numbufs;
	if (ss->outreadbuf == ss->inreadbuf) {
		/* Whoops, we're out of stuff to read.  Make a note and
		   wait for something to come in */
		ss->outreadbuf = -1;
		/* Disable the receiver if appropriate */
		if (ss->rxbufpolicy == POLICY_WHEN_FULL)
			ss->rxdisable = 1;
	}
	if (ss->inreadbuf < 0) {
		/* The interrupt handler has been waiting on us to free up a buffer
		   so it can input some data.  Let it know we have something it can
		   use.  */
		ss->inreadbuf = oldbuf;
	}
	spin_unlock_irqrestore(&ss->lock, flags);
	return(amnt);
}

/* No bigger than 32k for everything per tone zone */
#define MAX_SIZE 32768
/* No more than 30 subtones */
#define MAX_TONES 30

static int
ioctl_load_zone(struct tor_softc *ss, unsigned long data)
{
	struct tor_tone *samples[MAX_TONES];
	int next[MAX_TONES];
	struct tor_tone_def_header th;
	void *slab, *ptr;
	long size;
	struct tor_zone *z;
	struct tor_tone_def td;
	struct tor_tone *t;
	int x;
	int space;
	int res;
	/* XXX Unnecessary XXX */
	memset(samples, 0, sizeof(samples));
	/* XXX Unnecessary XXX */
	memset(next, 0, sizeof(next));
	copy_from_user(&th, (struct tor_tone_def_header *)data, sizeof(th));
	if ((th.count < 0) || (th.count > MAX_TONES))
		return -EINVAL;
	if ((th.size < 0) || (th.size > MAX_SIZE))
		return -EINVAL;
	space = size = sizeof(struct tor_zone) +
			th.count * sizeof(struct tor_tone) +
			th.size;
	if ((size > MAX_SIZE) || (size < 0))
		return -E2BIG;
	ptr = slab = (char *)kmalloc(size, GFP_KERNEL);
	if (!slab)
		return -ENOMEM;
	/* Zero it out for simplicity */
	memset(slab, 0, size);
	/* Grab the zone */
	z = (struct tor_zone *)slab;
	strncpy(z->name, th.name, sizeof(z->name) - 1);
	for (x=0;x<TOR_MAX_CADENCE;x++)
		z->ringcadence[x] = th.ringcadence[x];
	data += sizeof(struct tor_tone_def_header);
	ptr += sizeof(struct tor_zone);
	space -= sizeof(struct tor_zone);
	for (x=0;x<th.count;x++) {
		if (space < sizeof(struct tor_tone)) {
			/* Check space for tor_tone struct */
			kfree(slab);
			return -EINVAL;
		}
		if (copy_from_user(&td, (struct tor_tone_def *)data, sizeof(struct tor_tone_def))) {
			kfree(slab);
			return -EIO;
		}
		/* Index the current sample */
		samples[x] = t = (struct tor_tone *)ptr;
		/* Remember which sample is next */
		next[x] = td.next;
		/* Make sure the "next" one is sane */
		if ((next[x] >= th.count) || (next[x] < 0)) {
			kfree(slab);
			return -EINVAL;
		}
		if (td.tone >= TOR_TONE_MAX) {
			/* Make sure it's sane */
			kfree(slab);
			return -EINVAL;
		}
		/* Update pointers to account for tor_tone header */
		space -= sizeof(struct tor_tone);
		ptr += sizeof(struct tor_tone);
		data += sizeof(struct tor_tone_def);
		/* Fill in tonedata, datalen, and tonesamples fields */
		t->tonedata = (unsigned char *)(ptr);
		t->datalen = td.size;
		t->tonesamples = td.samples;
		t->next = NULL;					/* XXX Unnecessary XXX */
		/* Now some checks:  Enough space? */
		if (space < t->datalen) {
			kfree(slab);
			return -EINVAL;
		}
		/* Put actual tone data */
		if (copy_from_user(t->tonedata, (unsigned char *)data, t->datalen))
			return -EIO;
		if (!z->tones[td.tone])
			z->tones[td.tone] = t;
		/* Update pointers and space */
		data += t->datalen;
		ptr += t->datalen;
		space -= t->datalen;
	}
	for (x=0;x<th.count;x++) 
		/* Set "next" pointers */
		samples[x]->next = samples[next[x]];

	/* Actually register zone */
	res = tor_register_tone_zone(th.zone, z);
	if (res)
		kfree(slab);
	return res;
}

static inline void do_dtmf(int unit)
{
	char c;
	struct tor_tone *z;
	struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[unit];
	while (strlen(ss->txdialbuf)) {
		if (ss->digitmode == DIGIT_MODE_DTMF)
			z = dtmf_tones;
		else
			z = mfv1_tones;
		c = ss->txdialbuf[0];
		/* Skooch */
		memmove(ss->txdialbuf, ss->txdialbuf + 1, sizeof(ss->txdialbuf) - 1);
		switch(c) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			ss->curtone = z + (int)(c - '0');
			ss->tonep = 0;
			return;
		case '*':
			ss->curtone = z + 10;
			ss->tonep = 0;
			return;
		case '#':
			ss->curtone = z + 11;
			return;
		case 'A':
		case 'B':
		case 'C':
			ss->curtone = z + (c + 12 - 'A');
			ss->tonep = 0;
			return;
		case 'D':
			if (ss->digitmode == DIGIT_MODE_DTMF)
				ss->curtone = z + ( c + 12 - 'A');
			ss->tonep = 0;
			return;
		case 'a':
		case 'b':
		case 'c':
			ss->curtone = z + (c + 12 - 'a');
			ss->tonep = 0;
			return;
		case 'd':
			if (ss->digitmode == DIGIT_MODE_DTMF)
				ss->curtone = z + ( c + 12 - 'a');
			ss->tonep = 0;
			return;
		case 'T':
		case 't':
			ss->digitmode = DIGIT_MODE_DTMF;
			ss->tonep = 0;
			break;
		case 'M':
		case 'm':
			ss->digitmode = DIGIT_MODE_MFV1;
			ss->tonep = 0;
			break;
		default:
		}
	}
	/* Notify userspace process if there is nothing left */
	ss->dialing = 0;
	qevent(unit, TOR_EVENT_DIALCOMPLETE);
}

static int tor_first_empty_conference(void)
{
	/* Find first conference to which no one is subscribed */
	int x,y, inuse;
	for (y=NCONF-1;y>0;y--) {
		inuse=0;
		for (x=1;x<NTOR;x++)
			if (tor_softc[x].confn == y) {
				inuse++;
				break;
			}
		if (!inuse)
			return y;	
	}
	return -1;
}

/* device ioctl routine */
static	int
tor_ioctl(struct inode *inode, struct file *file, unsigned int cmd,
	unsigned long data)
{
int i,j,k,rv,ret;
u_char	c;
long	flags;
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[UNIT(file)];
struct	tor_params param;
struct	tor_maintinfo maint;
struct	tor_spaninfo span;
struct	tor_confinfo conf;
struct	tor_gains gain;
struct	tor_lineconfig lconf;
struct	tor_chanconfig cconf;
struct	tor_bufferinfo bi;
struct  tor_dialoperation tdo;
struct	tor_dialparams tdp;

	if(!data) return(-EINVAL);
	switch(cmd) {
	case	TOR_DIAL:
		if (copy_from_user(&tdo, (struct tor_dialoperation *)data, sizeof(tdo)))
			return -EIO;
		rv = 0;
		/* Force proper NULL termination */
		tdo.dialstr[TOR_MAX_DTMF_BUF - 1] = '\0';
		spin_lock_irqsave(&ss->lock, flags);
		switch(tdo.op) {
		case TOR_DIAL_OP_CANCEL:
			ss->curtone = NULL;
			ss->dialing = 0;
			ss->txdialbuf[0] = '\0';
			ss->tonep = 0;
			break;
		case TOR_DIAL_OP_REPLACE:
			strcpy(ss->txdialbuf, tdo.dialstr);
			ss->dialing = 1;
			do_dtmf(UNIT(file));
			break;
		case TOR_DIAL_OP_APPEND:
			if (strlen(tdo.dialstr) + strlen(ss->txdialbuf) >= TOR_MAX_DTMF_BUF)
				return -EBUSY;
			strncpy(ss->txdialbuf + strlen(ss->txdialbuf), tdo.dialstr, TOR_MAX_DTMF_BUF - strlen(ss->txdialbuf));
			ss->dialing = 1;
			do_dtmf(UNIT(file));
			break;
		default:
			rv = -EINVAL;
		}
		spin_unlock_irqrestore(&ss->lock, flags);
		return rv;
	case	TOR_GET_DIALPARAMS:
		tdp.dtmf_tonelen = dtmf_tones[0].tonesamples / 8;
		tdp.mfv1_tonelen = mfv1_tones[0].tonesamples / 8;
		tdp.reserved[0] = 0;
		tdp.reserved[1] = 0;
		tdp.reserved[2] = 0;
		tdp.reserved[3] = 0;
		if (copy_to_user((struct tor_dialparams *)data, &tdp, sizeof(tdp)))
			return -EIO;
		break;
	case	TOR_SET_DIALPARAMS:
		if (UNIT(file))
			return -EINVAL;
		if (copy_from_user(&tdp, (struct tor_dialparams *)data, sizeof(tdp)))
			return -EIO;
		if ((tdp.dtmf_tonelen > 4000) || (tdp.dtmf_tonelen < 10))
			return -EINVAL;
		if ((tdp.mfv1_tonelen > 4000) || (tdp.mfv1_tonelen < 10))
			return -EINVAL;
		for (i=0;i<16;i++)
			dtmf_tones[i].tonesamples = tdp.dtmf_tonelen * 8;
		dtmf_silence.tonesamples = tdp.dtmf_tonelen * 8;
		for (i=0;i<15;i++)
			mfv1_tones[i].tonesamples = tdp.mfv1_tonelen * 8;
		mfv1_silence.tonesamples = tdp.mfv1_tonelen * 8;
		/* Special case for K/P tone */
		mfv1_tones[10].tonesamples = tdp.mfv1_tonelen * 8 * 5 / 3;
		break;
	case	TOR_GET_BUFINFO:
		bi.rxbufpolicy = ss->rxbufpolicy;
		bi.txbufpolicy = ss->txbufpolicy;
		bi.numbufs = ss->numbufs;
		bi.bufsize = ss->blocksize;
		/* XXX FIXME! XXX */
		bi.readbufs = -1;
		bi.writebufs = -1;
		if (copy_to_user((struct tor_bufferinfo *)data, &bi, sizeof(bi)))
			return -EIO;
		break;
	case    TOR_SET_BUFINFO:
		if (copy_from_user(&bi, (struct tor_bufferinfo *)data, sizeof(bi)))
			return -EIO;
		if (bi.bufsize > TOR_MAX_BLOCKSIZE)
			return -EINVAL;
		if (bi.bufsize < 16)
			return -EINVAL;
		ss->rxbufpolicy = bi.rxbufpolicy & 0x1;
		ss->txbufpolicy = bi.txbufpolicy & 0x1;
		if ((rv = tor_reallocbufs(ss,  bi.bufsize, bi.numbufs)))
			return (rv);
		break;
	case	TOR_GET_BLOCKSIZE:  /* get blocksize */
		put_user(ss->blocksize,(int *)data); /* return block size */
		break;
	case	TOR_SET_BLOCKSIZE:  /* set blocksize */
		get_user(j,(int *)data);
		  /* cannot be larger than max amount */
		if (j > TOR_MAX_BLOCKSIZE) return(-EINVAL);
		  /* cannot be less then 16 */
		if (j < 16) return(-EINVAL);
		  /* allocate a single kernel buffer which we then
		     sub divide into four pieces */
		if ((rv = tor_reallocbufs(ss, j, ss->numbufs)))
			return (rv);
		break;
	case	TOR_AUDIOMODE:
		if (ss->sigtype != TOR_CLEAR) return (-EINVAL);
		get_user(j, (int *)data);
		if (j) 
			ss->flags |= AUDIO;
		else
			ss->flags &= ~AUDIO;
		break;
	case	TOR_GET_PARAMS: /* get channel timing parameters */
		  /* point to relevant structure */
		param.sigtype = ss->sigtype;  /* get signalling type */
		/* return non-zero if rx not in idle state */
		param.rxisoffhook = (ss->rxsig != q_sig[ss->sigtype]);          		  /* get various timers */
		param.prewinktime = ss->prewinktime; 
		param.preflashtime = ss->preflashtime;		
		param.winktime = ss->winktime;
		param.flashtime = ss->flashtime;
		param.starttime = ss->starttime;
		param.rxwinktime = ss->rxwinktime;
		param.rxflashtime = ss->rxflashtime;
		param.debouncetime = ss->debouncetime;
		copy_to_user((struct tor_params *)data,&param,sizeof(param));
		break;
	case	TOR_SET_PARAMS: /* set channel timing paramters */
		  /* point to relevant structure */
		copy_from_user(&param,(struct tor_params *)data,sizeof(param));
		/* NOTE: sigtype is *not* included in this */
		  /* get timing paramters */
		ss->prewinktime = param.prewinktime;
		ss->preflashtime = param.preflashtime;
		ss->winktime = param.winktime;
		ss->flashtime = param.flashtime;
		ss->starttime = param.starttime;
		ss->rxwinktime = param.rxwinktime;
		ss->rxflashtime = param.rxflashtime;
		ss->debouncetime = param.debouncetime;
		break;
	case	TOR_FLUSH:  /* flush input buffer, output buffer, and/or event queue */
		spin_lock_irqsave(&ss->lock, flags);
		get_user(i,(int *)data);  /* get param */
		if (i & TOR_FLUSH_READ)  /* if for read (input) */
		   {
			  /* initialize read buffers and pointers */
			ss->inreadbuf = 0;
			ss->outreadbuf = -1;
#if 0
			/* Do we need this? */
			ss->readn[0] = ss->readn[1] = 0;
			ss->readidx[0] = ss->readidx[1] = 0;
#endif
			wake_up_interruptible(&ss->readbufq);  /* wake_up_interruptible waiting on read */
			wake_up_interruptible(&ss->sel); /* wake_up_interruptible waiting on select */
		   }
		if (i & TOR_FLUSH_WRITE) /* if for write (output) */
		   {
			  /* initialize write buffers and pointers */
			ss->outwritebuf = -1;
			ss->inwritebuf = 0;
#if 0
			/* Do we ned this? */
			ss->writen[0] = ss->writen[1] = 0;
			ss->writeidx[0] = ss->writeidx[1] = 0;
#endif
			wake_up_interruptible(&ss->writebufq); /* wake_up_interruptible waiting on write */
			wake_up_interruptible(&ss->sel);  /* wake_up_interruptible waiting on select */
			   /* if IO MUX wait on write empty, well, this
				certainly *did* empty the write */
			if (ss->iomask & TOR_IOMUX_WRITEEMPTY)
				wake_up_interruptible(&ss->eventbufq); /* wake_up_interruptible waiting on IOMUX */
		   }
		if (i & TOR_FLUSH_EVENT) /* if for events */
		   {
			   /* initialize the event pointers */
			ss->eventinidx = ss->eventoutidx = 0;
		   }
		spin_unlock_irqrestore(&ss->lock, flags);
		break;
	case	TOR_SYNC:  /* wait for no tx */
		for(;;)  /* loop forever */
		   {
			spin_lock_irqsave(&ss->lock, flags);
			  /* Know if there is a write pending */
			i = (ss->outwritebuf > -1);
			spin_unlock_irqrestore(&ss->lock, flags);
			if (!i) break; /* skip if none */
			rv = schluffen(&ss->writebufq);
			if (rv) return(rv);
		   }
		break;
	case	TOR_GETEVENT:  /* Get event on queue */
		spin_lock_irqsave(&ss->lock, flags);
		  /* set up for no event */
		put_user(TOR_EVENT_NONE,(int *)data);
		  /* if some event in queue */
		if (ss->eventinidx != ss->eventoutidx)
		   {
			  /* get the data, bump index */
			put_user(ss->eventbuf[ss->eventoutidx++],(int *)data);
			  /* if index overflow, set to beginning */
			if (ss->eventoutidx >= TOR_MAX_EVENTSIZE)
				ss->eventoutidx = 0;
		   }		
		spin_unlock_irqrestore(&ss->lock, flags);
		break;
	case	TOR_IOMUX: /* wait for something to happen */
		get_user(ss->iomask,(int*)data);  /* save mask */
		if (!ss->iomask) return(-EINVAL);  /* cant wait for nothing */
		for(;;)  /* loop forever */
		   {
			  /* has to have SOME mask */
			ret = 0;  /* start with empty return value */
			spin_lock_irqsave(&ss->lock, flags);
			  /* if looking for read */
			if (ss->iomask & TOR_IOMUX_READ)
			   {
				/* if read available */
				if ((ss->outreadbuf > -1)  && !ss->rxdisable)
					ret |= TOR_IOMUX_READ;
			   }
			  /* if looking for write avail */
			if (ss->iomask & TOR_IOMUX_WRITE)
			   {
				if (ss->inwritebuf > -1)
					ret |= TOR_IOMUX_WRITE;
			   }
			  /* if looking for write empty */
			if (ss->iomask & TOR_IOMUX_WRITEEMPTY)
			   {
				  /* if everything empty -- be sure the transmitter is enabled */
				ss->txdisable = 0;
				if (ss->outwritebuf < 0)
					ret |= TOR_IOMUX_WRITEEMPTY;
			   }
			  /* if looking for signalling event */
			if (ss->iomask & TOR_IOMUX_SIGEVENT)
			   {
				  /* if event */
				if (ss->eventinidx != ss->eventoutidx)
					ret |= TOR_IOMUX_SIGEVENT;
			   }
			spin_unlock_irqrestore(&ss->lock, flags);
			  /* if something to return, or not to wait */
			if (ret || (ss->iomask & TOR_IOMUX_NOWAIT))
			   {
				  /* set return value */
				put_user(ret,(int *)data);
				break; /* get out of loop */
			   }
			rv = schluffen(&ss->eventbufq);
			if (rv) return(rv);
		   }
		  /* clear IO MUX mask */
		ss->iomask = 0;
		break;
	case	TOR_HOOK:  /* set hookswitch state */
		if (ss->flags & CLEAR) return (-EINVAL);
		get_user(j,(int *)data);
		/* Keep track of whether or not we're blocking */
		if (file->f_flags & O_NONBLOCK)
			ss->flags |= NONBLOCKING;
		else
			ss->flags &= ~NONBLOCKING;
		  /* if not in right state, return busy */
		if ((j == TOR_RINGOFF) && 
		    (ss->txstate != TOR_TXSTATE_RINGON) &&
			(ss->txstate != TOR_TXSTATE_RINGOFF)) return(-EBUSY);
		  /* if not in right state, return busy */
		if ((j != TOR_ONHOOK) && (j != TOR_OFFHOOK) &&
		   (j != TOR_RINGOFF) &&
		    (ss->txstate != TOR_TXSTATE_ONHOOK) &&
		      (ss->txstate != TOR_TXSTATE_OFFHOOK) &&
			(ss->txstate != TOR_TXSTATE_DEBOUNCE) &&
			  (ss->txstate != TOR_TXSTATE_KEWL)) return (-EBUSY);
		switch(j) 
		   {
		    case TOR_ONHOOK:  /* put on hook */
			spin_lock_irqsave(&ss->lock, flags);
			if (ss->sigtype == TOR_FXOKS)
			   {
				tor_sethook(UNIT(file),TOR_TXSIG_KEWL);
				ss->otimer = TOR_KEWLTIME;
				ss->txstate = TOR_TXSTATE_KEWL;
			   }
			else
			   {
				tor_sethook(UNIT(file),TOR_TXSIG_ONHOOK);
				ss->otimer = 0; /* clear timer */
				ss->txstate = TOR_TXSTATE_ONHOOK;
			   }
			spin_unlock_irqrestore(&ss->lock, flags);
			break;
		    case TOR_OFFHOOK:  /* take off hook */
			if (ss->txstate == TOR_TXSTATE_KEWL) return (-EBUSY);
			spin_lock_irqsave(&ss->lock, flags);
			tor_sethook(UNIT(file),TOR_TXSIG_OFFHOOK);
			ss->otimer = ss->debouncetime; /* set for debounce timeout */
			ss->txstate = TOR_TXSTATE_DEBOUNCE;
			spin_unlock_irqrestore(&ss->lock, flags);
			break;
		    case TOR_START:
			   /* has to be onhook for this to work */
			if (ss->txstate != TOR_TXSTATE_ONHOOK) return (-EBUSY);
			spin_lock_irqsave(&ss->lock, flags);
			tor_sethook(UNIT(file),TOR_TXSIG_START);
			if ((ss->sigtype == TOR_FXOGS) || (ss->sigtype == TOR_FXOLS) ||
			    (ss->sigtype == TOR_FXOKS)) {
				ss->txstate = TOR_TXSTATE_RINGON;
				ss->otimer = 0;
				if (ss->curzone) {
					ss->cadencepos = 0;
					ss->otimer = ss->curzone->ringcadence[0];
				}
				if (!ss->otimer)
					ss->otimer = ss->starttime;  /* initialize timer */
			} else {
				ss->txstate = TOR_TXSTATE_START;
				ss->otimer = ss->starttime;  /* initialize timer */
			}
			spin_unlock_irqrestore(&ss->lock, flags);
			/* If we're not supposed to block, don't ;-) */
			if (file->f_flags & O_NONBLOCK) 
				return -EINPROGRESS;
			rv = schluffen(&ss->txstateq);
			if (rv) return rv;
			break;
		    case TOR_WINK:  /* wink */
			   /* has to be onhook for this to work */
			if (ss->txstate != TOR_TXSTATE_ONHOOK) return (-EBUSY);
			spin_lock_irqsave(&ss->lock, flags);
			ss->otimer = ss->prewinktime;  /* initialize timer */
			ss->txstate = TOR_TXSTATE_PREWINK;
			spin_unlock_irqrestore(&ss->lock, flags);
			/* If we're not supposed to block, don't ;-) */
			if (file->f_flags & O_NONBLOCK) 
				return -EINPROGRESS;
			rv = schluffen(&ss->txstateq);
			if (rv) return rv;
			break;
		    case TOR_FLASH:  /* flash */
			   /* has to be offhook for this to work */
			if (ss->txstate != TOR_TXSTATE_OFFHOOK) return (-EBUSY);
			spin_lock_irqsave(&ss->lock, flags);
			ss->otimer = ss->preflashtime; /* initialize timer */
			ss->txstate = TOR_TXSTATE_PREFLASH;
			spin_unlock_irqrestore(&ss->lock, flags);
			/* If we're not supposed to block, don't ;-) */
			if (file->f_flags & O_NONBLOCK) 
				return -EINPROGRESS;
			rv = schluffen(&ss->txstateq);
			if (rv) return rv;
			break;
		    case TOR_RINGOFF:  /* take out of ring state */
			spin_lock_irqsave(&ss->lock, flags);
			tor_sethook(UNIT(file),TOR_TXSIG_ONHOOK);
			ss->otimer = 0; /* clear timer */
			ss->txstate = TOR_TXSTATE_ONHOOK;
			spin_unlock_irqrestore(&ss->lock, flags);
			break;
		    default:	
			break;
		   }
		break;
	case	TOR_MAINT:  /* do maintence stuff */
		  /* must only do this on master device */
		if (UNIT(file)) return(-EINVAL);
		  /* get struct from user */
		if (copy_from_user(&maint,(struct tor_maintinfo *) data,
			sizeof(maint))) return -EIO;
		  /* must be valid span number */
		if ((maint.spanno < 1) || (maint.spanno > 2)) return(-EINVAL);
		spin_lock_irqsave(&ss->lock, flags);
		  /* save current maint state */
		i = maints[maint.spanno - 1];
		  /* set maint mode */
		maints[maint.spanno - 1] = maint.command;
		switch(maint.command)
		   {
		    case TOR_MAINT_NONE:
			  /* if same, ignore it */
			if (i == TOR_MAINT_NONE) break;
			t1out(maint.spanno,0x37,0x8c); /* clear system */
			break;
		    case TOR_MAINT_LOCALLOOP:
			  /* if same, ignore it */
			if (i == TOR_MAINT_LOCALLOOP) break;
			t1out(maint.spanno,0x37,0xcc); /* local loopback */
			break;
		    case TOR_MAINT_REMOTELOOP:
			  /* if same, ignore it */
			if (i == TOR_MAINT_REMOTELOOP) break;
			t1out(maint.spanno,0x37,0x9c); /* remote loopback */
			break;
		    case TOR_MAINT_LOOPUP:
			t1out(maint.spanno,0x30,2); /* send loopup code */
			mainttimer[maint.spanno - 1] = TOR_LOOPCODE_TIME;
			spin_unlock_irqrestore(&ss->lock, flags);
			rv = schluffen(&maintq[maint.spanno - 1]);
			if (rv) return rv;
			spin_lock_irqsave(&ss->lock, flags);
			break;
		    case TOR_MAINT_LOOPDOWN:
			t1out(maint.spanno,0x30,4); /* send loopdown code */
			mainttimer[maint.spanno - 1] = TOR_LOOPCODE_TIME;
			spin_unlock_irqrestore(&ss->lock, flags);
			rv = schluffen(&maintq[maint.spanno - 1]);
			if (rv) return rv;
			spin_lock_irqsave(&ss->lock, flags);
			break;
		   }
		alarm_notify();  /* process alarm-related events */
		spin_unlock_irqrestore(&ss->lock, flags);
		break;
	case	TOR_SPANSTAT:  /* get alarm stuff */
		copy_from_user(&span,(struct tor_spaninfo *) data,sizeof(span));
		i = span.spanno; /* get specified span number */
		if ((i < 0) || (i > 2)) return(-EINVAL);  /* if bad span no */
		if (i == 0) /* if to figure it out for this chan */
		   {
			  /* cant do this for master */
			if (!UNIT(file)) return(-EINVAL);
			  /* calculate it */
			i = ((UNIT(file) - 1) / 24) + 1;
		   }
		span.spanno = i; /* put the span # in here */
		span.alarms = alarms[i - 1];
		spin_lock_irqsave(&ss->lock, flags);
		t1out(i,0x31,0xff);  
		c = t1in(i,0x31); /* get RIR2 */
		span.bpvcount = bpvcounts[i - 1];
		spin_unlock_irqrestore(&ss->lock, flags);
		span.rxlevel = c >> 6;  /* get rx level */
		span.txlevel = txlevels[i - 1];  /* get tx level */
		span.syncsrc = syncsrc;  /* get active sync source */
		span.numchans = 0;
		for (j=(i-1) * 24 + 1; j <= (i*24); j++)
			if (tor_softc[j].sigtype)
				span.numchans++;
		copy_to_user((struct tor_spaninfo *) data,&span,sizeof(span));
		break;
	case	TOR_GETCONF:  /* get conf stuff */
		if (!(ss->flags & AUDIO)) return (-EINVAL);
		copy_from_user(&conf,(struct tor_confinfo *) data,sizeof(conf));
		i = conf.chan;  /* get channel no */
		   /* if zero, use current channel no */
		if (!i) i = UNIT(file);
		  /* make sure channel number makes sense */
		if ((i < 0) || (i > NTOR)) return(-EINVAL);
		conf.chan = i;  /* get channel number */
		conf.confno = tor_softc[i].confn;  /* get conference number */
		conf.confmode = tor_softc[i].confmode; /* get conference mode */
		copy_to_user((struct tor_confinfo *) data,&conf,sizeof(conf));
		break;
	case	TOR_SETCONF:  /* set conf stuff */
		if (!(ss->flags & AUDIO)) return (-EINVAL);
		copy_from_user(&conf,(struct tor_confinfo *) data,sizeof(conf));
		i = conf.chan;  /* get channel no */
		   /* if zero, use current channel no */
		if (!i) i = UNIT(file);
		  /* make sure channel number makes sense */
		if ((i < 1) || (i > NTOR)) return(-EINVAL);
		if ((conf.confmode & TOR_CONF_MODE_MASK) < 4) {
			/* Monitor mode -- it's a channel */
			if ((conf.confno < 0) || (conf.confno > NTOR)) return(-EINVAL);
		} else {
			  /* make sure conf number makes sense, too */
			if ((conf.confno < -1) || (conf.confno > NCONF)) return(-EINVAL);
		}
			
		  /* if taking off of any conf, must have 0 mode */
		if ((!conf.confno) && conf.confmode) return(-EINVAL);
		conf.chan = i;  /* return with real channel # */
		spin_lock_irqsave(&ss->lock, flags);
		if (conf.confno == -1)
			conf.confno = tor_first_empty_conference();
		if ((conf.confno < 1) && (conf.confmode)) {
			/* No more empty conferences */
			spin_unlock_irqrestore(&ss->lock, flags);
			return -EBUSY;
		}
		  /* if changing confs, clear last added info */
		if (conf.confno != tor_softc[i].confn) 
			tor_softc[i].conflast = tor_softc[i].conflast1 = 0;
		tor_softc[i].confn = conf.confno;   /* set conference number */
		tor_softc[i].confmode = conf.confmode;  /* set conference mode */
		copy_to_user((struct tor_confinfo *) data,&conf,sizeof(conf));
		spin_unlock_irqrestore(&ss->lock, flags);
		break;
	case	TOR_CONFLINK:  /* do conf link stuff */
		if (!(ss->flags & AUDIO)) return (-EINVAL);
		copy_from_user(&conf,(struct tor_confinfo *) data,sizeof(conf));
		  /* check sanity of arguments */
		if ((conf.chan < 0) || (conf.chan > NCONF)) return(-EINVAL);
		if ((conf.confno < 0) || (conf.confno > NCONF)) return(-EINVAL);
		  /* cant listen to self!! */
		if (conf.chan && (conf.chan == conf.confno)) return(-EINVAL);
		spin_lock_irqsave(&ss->lock, flags);
		  /* if to clear all links */
		if ((!conf.chan) && (!conf.confno))
		   {
			   /* clear all the links */
			memset(conf_links,0,sizeof(conf_links));
			spin_unlock_irqrestore(&ss->lock, flags);
			break;
		   }
		rv = 0;  /* clear return value */
		/* look for already existant specified combination */
		for(i = 1; i <= NCONF; i++)
		   {
			  /* if found, exit */
			if ((conf_links[i].src == conf.chan) &&
				(conf_links[i].dst == conf.confno)) break;
		   }
		if (i <= NCONF) /* if found */
		   {
			if (!conf.confmode) /* if to remove link */
			   {
				conf_links[i].src = conf_links[i].dst = 0;
			   }
			else /* if to add and already there, error */
			   {
				rv = -EEXIST;
			   }
		   }
		else  /* if not found */
		   {
			if (conf.confmode) /* if to add link */
			   {
				/* look for empty location */
				for(i = 1; i <= NCONF; i++)
				   {
					  /* if empty, exit loop */
					if ((!conf_links[i].src) &&
						 (!conf_links[i].dst)) break;
				   }
				   /* if empty spot found */
				if (i <= NCONF)
				   {
					conf_links[i].src = conf.chan;
					conf_links[i].dst = conf.confno;
				   }
				else /* if no empties -- error */
				   {
					rv = -ENOSPC;
				   }
			   }
			else /* if to remove, and not found -- error */
			   {
				rv = -ENOENT;
			   }
		   }
		spin_unlock_irqrestore(&ss->lock, flags);
		return(rv);
	case	TOR_CONFDIAG:  /* output diagnostic info to console */
		if (!(ss->flags & AUDIO)) return (-EINVAL);
		get_user(j,(int *)data);  /* get conf # */
		  /* loop thru the interesting ones */
		for(i = ((j) ? j : 1); i <= ((j) ? j : NCONF); i++)
		   {
			c = 0;
			for(k = 1; k < NTOR; k++)
			   {
				  /* skip if not in this conf */
				if (tor_softc[k].confn != i) continue;
				if (!c) printk("Conf #%d:\n",i);
				c = 1;
				printk("chan %d, mode %x\n",
					k,tor_softc[k].confmode);
			   }
			rv = 0;
			for(k = 1; k <= NCONF; k++)
			   {
				if (conf_links[k].dst == i)
				   {
					if (!c) printk("Conf #%d:\n",i);
					c = 1;
					if (!rv) printk("Snooping on:\n");
					rv = 1;
					printk("conf %d\n",conf_links[k].src);
				   }
			   }
			if (c) printk("\n");
		   }
		break;
	case	TOR_GETGAINS:  /* get gain stuff */
		if (!(ss->flags & AUDIO)) return (-EINVAL);
		if (copy_from_user(&gain,(struct tor_gains *) data,sizeof(gain)))
			return -EIO;
		i = gain.chan;  /* get channel no */
		   /* if zero, use current channel no */
		if (!i) i = UNIT(file);
		  /* make sure channel number makes sense */
		if ((i < 0) || (i > NTOR)) return(-EINVAL);
		gain.chan = i; /* put the span # in here */
		for (j=0;j<256;j++)  {
			gain.txgain[j] = txgainx[i][j];
			gain.rxgain[j] = rxgainx[i][j];
		}
		if (copy_to_user((struct tor_gains *) data,&gain,sizeof(gain)))
			return -EIO;
		break;
	case	TOR_SETGAINS:  /* set gain stuff */
		if (!(ss->flags & AUDIO)) return (-EINVAL);
		if (copy_from_user(&gain,(struct tor_gains *) data,sizeof(gain)))
			return -EIO;
		i = gain.chan;  /* get channel no */
		   /* if zero, use current channel no */
		if (!i) i = UNIT(file);
		  /* make sure channel number makes sense */
		if ((i < 0) || (i > NTOR)) return(-EINVAL);
		gain.chan = i; /* put the span # in here */
		for (j=0;j<256;j++) {
			rxgainx[i][j] = gain.rxgain[j];
			txgainx[i][j] = gain.txgain[j];
		}
		break;
	case TOR_CHANCONFIG: /* set channel configuration */
		if (UNIT(file)) return(-EINVAL); /* can only be done from master unit */
		copy_from_user(&cconf,(struct tor_chanconfig *) data,
			sizeof(cconf));
		  /* make sure channel number makes sense */
		if ((cconf.chan > 48) && (!(cconf.sigtype & TOR_PSEUDO))) return(-EINVAL);
		if ((cconf.chan <= 48) && (cconf.sigtype & TOR_PSEUDO)) return(-EINVAL);
		if ((cconf.chan < 1) || (cconf.chan > NTOR)) return(-EINVAL);
#ifndef CONFIG_TORMENTA_NET
		if ((cconf.sigtype == TOR_HDLCNET)) return (-ENOSYS);
#endif		
		if ((cconf.sigtype == TOR_SLAVE) && ((cconf.master > 48) || (cconf.master < 1))) return (-EINVAL);
		/* Keep them from bundling non-clear channel or HDLC channels together */
		if ((cconf.sigtype == TOR_SLAVE) && !(ss[cconf.master].flags & CLEAR)) return (-EINVAL);
		return tor_chanconfig(cconf.chan,cconf.sigtype, cconf.master);
		break;
	case TOR_SHUTDOWN:
		if (UNIT(file)) return(-EINVAL); /* can only be done from master unit */
		tor_shutdown();
		break;
	case TOR_DEFAULTZONE:
		if (UNIT(file)) return(-EINVAL); /* can only be done from master unit */
		get_user(j,(int *)data);  /* get conf # */
		if ((j < 0) || (j >= TOR_TONE_ZONE_MAX)) return (-EINVAL);
		spin_lock(&zone_lock);
		default_zone = j;
		spin_unlock(&zone_lock);
		break;
	case TOR_LOADZONE:
		rv = ioctl_load_zone(ss, data);
		return rv;
	case TOR_FREEZONE:
		get_user(j,(int *)data);  /* get conf # */
		if ((j < 0) || (j >= TOR_TONE_ZONE_MAX)) return (-EINVAL);
		free_tone_zone(j);
		break;
	case TOR_STARTUP: /* set lines configuration and start system */
		if (UNIT(file)) return(-EINVAL); /* can only be done from master unit */
		copy_from_user(&lconf,(struct tor_lineconfig *) data,
			sizeof(lconf));
		tor_startup(&lconf);
		break;
	case TOR_CONFMUTE:  /* set confmute flag */
		if (!(ss->flags & AUDIO)) return (-EINVAL);
		get_user(j,(int *)data);  /* get conf # */
		ss->confmute = j;
		break;
	case TOR_SETTONEZONE:
		get_user(j,(int *)data);
		spin_lock_irqsave(&ss->lock, flags);
		rv =  set_tone_zone(ss, j);
		spin_unlock_irqrestore(&ss->lock, flags);
		return rv;
	case TOR_GETTONEZONE:
		spin_lock_irqsave(&ss->lock, flags);
		if (ss->curzone)
			rv = ss->tonezone;
		else
			rv = default_zone;
		spin_unlock_irqrestore(&ss->lock, flags);
		put_user(rv,(int *)data); /* return block size */
		break;
	case TOR_SENDTONE:
		get_user(j,(int *)data);
		spin_lock_irqsave(&ss->lock, flags);
		rv = start_tone(ss, j);	
		spin_unlock_irqrestore(&ss->lock, flags);
		return rv;
#ifdef CONFIG_TORMENTA_ECHOCANCEL
	case TOR_ECHOCANCEL:
		if (!(ss->flags & AUDIO))
			return -EINVAL;
		if (ss->flags & PSEUDO)
			return -EINVAL;
		get_user(j, (int *)data);
		if (j) {
			ss->echocancel = 1;
			init_cc(&ss->ec);
		} else
			ss->echocancel = 0;
		break;
#else
	case TOR_ECHOCANCEL:
		return -ENOSYS;
#endif		
	default:
		printk("Unknown IOCTL: %d!\n", cmd);
		return -ENOTTY;
	}
	return 0;
}

/* device poll routine */
static unsigned int
tor_poll(struct file *file, struct poll_table_struct *wait_table)
{   
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[UNIT(file)];
int	ret;
long flags;
	  /* do the poll wait */
	poll_wait(file, &ss->sel, wait_table);
	ret = 0; /* start with nothing to return */
	spin_lock_irqsave(&ss->lock, flags);
	   /* if at least 1 write buffer avail */
	if (ss->inwritebuf > -1) {
		ret |= POLLOUT | POLLWRNORM;
	}
	if ((ss->outreadbuf > -1) && !ss->rxdisable) {
		ret |= POLLIN | POLLRDNORM;
	}
	if (ss->eventoutidx != ss->eventinidx)
	   {
		/* Indicate an exception */
		ret |= POLLPRI;
	   }
	spin_unlock_irqrestore(&ss->lock, flags);
	return(ret);  /* return what we found */
}


/* set hookswitch state routine */
static void tor_sethook(int chan, int what)
{
int	i,k,n,b;
u_char	m,c;
long flags;

static int outs[8][4] = {
	{ 0, 0, 0, 0 },  /* no sig */
	{ 0, TOR_ABIT | TOR_BBIT,TOR_ABIT | TOR_BBIT, 0 },  /* E and M */
	{ TOR_BBIT, TOR_ABIT | TOR_BBIT, TOR_ABIT | TOR_BBIT, 0 }, /* FXS Loopstart */
	{ TOR_BBIT, TOR_ABIT | TOR_BBIT, TOR_ABIT, 0 }, /* FXS Groundstart */
	{ TOR_BBIT, TOR_ABIT | TOR_BBIT, TOR_ABIT | TOR_BBIT, 0 }, /* FXS Kewlstart */
	{ TOR_BBIT, TOR_BBIT, 0, 0 }, /* FXO Loopstart */
	{ TOR_ABIT | TOR_BBIT, TOR_BBIT, 0, 0 }, /* FXO Groundstart */
	{ TOR_BBIT, TOR_BBIT, 0, TOR_ABIT | TOR_BBIT } /* FXO Kewlstart */
	} ;

	  /* ignore if a pseudo-device */
	if (tor_softc[chan].flags & PSEUDO) return;
	  /* make sure its valid */
	if ((chan < 1) || (chan > NTOR)) return;
	i = (chan - 1);
	k = (i / 24); /* get span number */
	n = (i % 24);  /* get channel number */
	b = (n / 8); /* get byte number */
	m = 1 << (n & 7); /* get mask */
	c = txsigs[k][b];
	c &= ~m;  /* clear mask bit */
	  /* set mask bit, if bit is to be set */
	if (outs[tor_softc[chan].sigtype][what] & TOR_ABIT) c |= m;
	txsigs[k][b] = c;
	spin_lock_irqsave(&tor_softc[chan].lock, flags);
	t1out(k + 1,0x70 + b,c);
	t1out(k + 1,0x76 + b,c);
	b += 3; /* now points to b bit stuff */
	  /* get current signalling values */
	c = txsigs[k][b];
	c &= ~m;  /* clear mask bit */
	  /* set mask bit, if bit is to be set */
	if (outs[tor_softc[chan].sigtype][what] & TOR_BBIT) c |= m;
	  /* save new signalling values */
	txsigs[k][b] = c;
	  /* output them into the chip */
	t1out(k + 1,0x70 + b,c);
	t1out(k + 1,0x76 + b,c);
	spin_unlock_irqrestore(&tor_softc[chan].lock, flags);
	return;
}

/* process change in receive signalling */
static void rxchange(int chan, u_char cursig)
{
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[chan];

	   /* if A bit has changed */
	if ((ss->rxsig & TOR_ABIT) != (cursig & TOR_ABIT))
	   {
		switch(ss->sigtype)
		   {
		    case TOR_EM:  /* E and M */
			if (cursig & TOR_ABIT)  /* went off hook */
			   {
				   /* set wink timer */
				ss->itimer = ss->rxwinktime;
			   }
			else /* went on hook */
			   {
				if (ss->itimer)
				    qevent(chan,TOR_EVENT_WINKFLASH); 
				else
				   {
				    qevent(chan,TOR_EVENT_ONHOOK); 
				    ss->gotgs = 0;
				   }
				ss->itimer = 0;
			   }
			break;
		   case TOR_FXSKS:  /* FXS Kewlstart */
			  /* ignore a bit poopy if loop no closed and stable */
			if (ss->txstate != TOR_TXSTATE_OFFHOOK) break;
			/* fall through intentionally */
		   case TOR_FXSGS:  /* FXS Groundstart */
			if (cursig & TOR_ABIT) /* if went on hook */
			   {
				  /* if not during offhook debounce time */
				if (ss->txstate != TOR_TXSTATE_DEBOUNCE)
				   {
					ss->gotgs = 0;
					qevent(chan,TOR_EVENT_ONHOOK);
				   }
			   }
			break;
		   case TOR_FXOLS: /* FXO Loopstart */
		   case TOR_FXOGS: /* FXO Groundstart */
		   case TOR_FXOKS: /* FXO Kewlstart */
			if (cursig & TOR_ABIT) /* if went off hook */
			   {
				  /* if asserting ring, stop it */
				if (ss->txstate == TOR_TXSTATE_START)
				   {
					tor_sethook(chan,TOR_TXSIG_OFFHOOK);
					ss->otimer = TOR_AFTERSTART_TIME;
					ss->txstate = TOR_TXSTATE_AFTERSTART;
				   }
				if (ss->itimer) /* if timer still running */
				    qevent(chan,TOR_EVENT_WINKFLASH); 
				else
				   {
					  /* if havent got GS detect */
					if (!ss->gotgs)
					   {
						qevent(chan,TOR_EVENT_RINGOFFHOOK); 
						ss->gotgs = 1;
						ss->itimer = 0;
					   }
				   }
				ss->itimer = 0;
			   }
			if (!(cursig & TOR_ABIT)) /* if went on hook */
			   {
				  /* if not during offhook debounce time */
				if ((ss->txstate != TOR_TXSTATE_DEBOUNCE) &&
				    (ss->txstate != TOR_TXSTATE_KEWL))
				   {
					ss->itimer = ss->rxflashtime;
				   }
			   }
			break;
		    default:
			break;
		   }
	   }
	   /* if B bit has changed */
	if ((ss->rxsig & TOR_BBIT) != (cursig & TOR_BBIT))
	   {
		switch(ss->sigtype)
		   {
		   case TOR_FXSLS:  /* FXS Loopstart */
		   case TOR_FXSGS:  /* FXS Groundstart */
		   case TOR_FXSKS:  /* FXS Kewlstart */
			if (cursig & TOR_BBIT) /* if trailing edge of ring */
				qevent(chan,TOR_EVENT_RINGOFFHOOK);
			break;
		   case TOR_FXOGS: /* FXO Groundstart */
			if (!(cursig & TOR_BBIT)) /* if ground detected */
			   {
				  /* if havent got gs, report it */
				if (!ss->gotgs)
				   {
					qevent(chan,TOR_EVENT_RINGOFFHOOK);
					ss->gotgs = 1;
				   }
			   }
			break;
		    default:
			break;
		   }
	   }
	return;
}

/* process input time-out */
static void do_itimer(int chan)
{
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[chan];

	/* the only way this could have gotten here, is if a channel
	    went off hook longer then the wink or flash detect timeout */

	switch(ss->sigtype)
	   {
	    case TOR_FXOLS:  /* if FXO, its definitely on hook */
	    case TOR_FXOGS:
	    case TOR_FXOKS:
		qevent(chan,TOR_EVENT_ONHOOK);
		ss->gotgs = 0; 
		break;
	    default:  /* otherwise, its definitely off hook */
		qevent(chan,TOR_EVENT_RINGOFFHOOK); 
		break;
	   }
}

static void do_otimer(int chan)
{
struct	tor_softc	*ss = (struct tor_softc *)&tor_softc[chan];
	int len=0;

	switch(ss->txstate)
	   {
	   	case TOR_TXSTATE_RINGOFF:
		tor_sethook(chan, TOR_TXSIG_START);
		if (ss->curzone) {
			++ss->cadencepos;
			if (ss->cadencepos >= TOR_MAX_CADENCE)
				ss->cadencepos = 0;
			len = ss->curzone->ringcadence[ss->cadencepos];

			if (!len) {
				ss->cadencepos = 0;
				len = ss->curzone->ringcadence[ss->cadencepos];
			}

		} else
			len = ss->starttime;
		ss->otimer = len;
		ss->txstate = TOR_TXSTATE_RINGON;
		qevent(chan, TOR_EVENT_RINGERON);
		break;
	   	case TOR_TXSTATE_RINGON:
		tor_sethook(chan, TOR_TXSIG_OFFHOOK);
		if (ss->curzone) {
			++ss->cadencepos;
			if (ss->cadencepos >= TOR_MAX_CADENCE)
				ss->cadencepos = 0;
			len = ss->curzone->ringcadence[ss->cadencepos];

			if (!len) {
				ss->cadencepos = 0;
				len = ss->curzone->ringcadence[ss->cadencepos];
			}

		} else
			len = TOR_RINGOFFTIME;
		ss->otimer = len;
		ss->txstate = TOR_TXSTATE_RINGOFF;
		qevent(chan, TOR_EVENT_RINGEROFF);
		wake_up_interruptible(&ss->txstateq);
		break;
	    case TOR_TXSTATE_START:  /* was starting */
		tor_sethook(chan,TOR_TXSIG_OFFHOOK);
		ss->otimer = TOR_AFTERSTART_TIME;
		ss->txstate = TOR_TXSTATE_AFTERSTART;
		wake_up_interruptible(&ss->txstateq);
		break;
	    case TOR_TXSTATE_PREWINK:  /* was pre-wink */
		tor_sethook(chan,TOR_TXSIG_OFFHOOK);
		ss->otimer = ss->winktime;
		ss->txstate = TOR_TXSTATE_WINK;
		break;
	    case TOR_TXSTATE_WINK:  /* was wink */
		tor_sethook(chan,TOR_TXSIG_ONHOOK);
		ss->otimer = 0;
		ss->txstate = TOR_TXSTATE_ONHOOK;
		if (ss->flags & NONBLOCKING)
			qevent(chan, TOR_EVENT_HOOKCOMPLETE);
		wake_up_interruptible(&ss->txstateq);
		break;
	    case TOR_TXSTATE_PREFLASH:  /* was pre-flash */
		tor_sethook(chan,TOR_TXSIG_ONHOOK);
		ss->otimer = ss->flashtime;
		ss->txstate = TOR_TXSTATE_FLASH;
		break;
	    case TOR_TXSTATE_FLASH:  /* was flash */
		tor_sethook(chan,TOR_TXSIG_OFFHOOK);
		ss->otimer = 0;
		ss->txstate = TOR_TXSTATE_OFFHOOK;
		if (ss->flags & NONBLOCKING)
			qevent(chan, TOR_EVENT_HOOKCOMPLETE);
		wake_up_interruptible(&ss->txstateq);
		break;
	    case TOR_TXSTATE_DEBOUNCE:  /* was debounce */
		ss->otimer = 0;
		ss->txstate = TOR_TXSTATE_OFFHOOK;
		break;
	    case TOR_TXSTATE_AFTERSTART:  /* was after start */
		ss->otimer = 0;
		ss->txstate = TOR_TXSTATE_OFFHOOK;
		if (ss->flags & NONBLOCKING)
			qevent(chan, TOR_EVENT_HOOKCOMPLETE);
		wake_up_interruptible(&ss->txstateq);
		break;
	    case TOR_TXSTATE_KEWL:  /* was after on hook */
		tor_sethook(chan,TOR_TXSIG_ONHOOK);
		ss->otimer = 0;
		ss->txstate = TOR_TXSTATE_ONHOOK;
		break;
	    default:
		break;
	   }
	return;
}


static inline unsigned char torintr_transmit(int unit)
{
	/* Handle transmission for a given unit */
	struct tor_softc *ss = &tor_softc[unit];
	/* We transmit data from our master channel */
	struct tor_softc *ms = &tor_softc[ss->master];
	/* ulaw-encoded transmit character */
	unsigned char txc;
	/* Buffer we're using */
	unsigned char *buf;
	/* Old buffer number */
	int oldbuf;
	/* Linear representation */
	int txlin;
	/* and how about another int */
	int k;

	/* Let's pick something to transmit.  First source to
	   try is our write-out buffer.  Always check it first because
	   its our 'fast path' for whatever that's worth. */
	if ((ms->outwritebuf > -1) && !ms->txdisable) {
		buf= ms->writebuf[ms->outwritebuf];
		if (ms->flags & HDLC) {
			/* If this is an HDLC channel we only send a byte of
			   HDLC. */
			if (ms->txhdlc.bits < 8)
				/* Load a byte of data only if needed */
				fasthdlc_tx_load_nocheck(&ms->txhdlc, buf[ms->writeidx[ms->outwritebuf]++]);
			txc = fasthdlc_tx_run_nocheck(&ms->txhdlc);
		} else {
			txc = buf[ms->writeidx[ms->outwritebuf]++];
		}
		/* Check buffer status */
		if (ms->writeidx[ms->outwritebuf] >= ms->writen[ms->outwritebuf]) {
			/* We've reached the end of our buffer.  Go to the next. */
			oldbuf = ms->outwritebuf;
			/* Clear out write index and such */
			ms->writeidx[oldbuf] = 0;
			ms->writen[oldbuf] = 0;
			ms->outwritebuf = (ms->outwritebuf + 1) % ms->numbufs;
			if (ms->outwritebuf == ms->inwritebuf) {
				/* Whoopsies, we're run out of buffers.  Mark ours
				   as -1 and wait for the filler to notify us that
				   there is something to write */
				ms->outwritebuf = -1;
				if (ms->iomask & (TOR_IOMUX_WRITE | TOR_IOMUX_WRITEEMPTY))
						wake_up_interruptible(&ms->eventbufq);
				/* If we're only supposed to start when full, disable the transmitter */
				if (ms->txbufpolicy == POLICY_WHEN_FULL)
					ms->txdisable = 1;
			}
			if (ms->inwritebuf < 0) {
				/* The filler doesn't have a place to put data.  Now
				   that we're done with this buffer, notify them. */
				ms->inwritebuf = oldbuf;
				if (!(ms->flags & NETDEV)) {
					wake_up_interruptible(&ms->writebufq);
					wake_up_interruptible(&ms->sel);
					if (ms->iomask & TOR_IOMUX_WRITE)
						wake_up_interruptible(&ms->eventbufq);
				}
			}
			/* Transmit a flag if this is an HDLC channel */
			if (ms->flags & HDLC)
				fasthdlc_tx_frame_nocheck(&ms->txhdlc);
#ifdef CONFIG_TORMENTA_NET
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
			/* Make sure the NET bh runs */
			mark_bh(NET_BH);
#else
			netif_wake_queue(&ms->netdev.netdev);
#endif
#endif
		}
	} else if (ms->curtone) {
		/* Pick our default value from the next sample of the current tone */
		txc = ms->curtone->tonedata[ms->tonep++ % ms->curtone->datalen];
		if (ms->tonep >= ms->curtone->tonesamples) {
			/* Go to the next sample of the tone */
			ms->tonep = 0;
			ms->curtone = ms->curtone->next;
			if (!ms->curtone) {
				/* No more tones...  Is this dtmf or mf?  If so, go to the next digit */
				if (ms->dialing)
					do_dtmf(ss->master);
			}
		}
	} else if (ms->flags & HDLC) {
		/* Okay, if we're HDLC, then transmit a flag by default */
		if (ms->txhdlc.bits < 8) 
			fasthdlc_tx_frame_nocheck(&ms->txhdlc);
		txc = fasthdlc_tx_run_nocheck(&ms->txhdlc);
	} else txc = 0x7f;	/* Lastly we use 0x7f on telephony channels */
	
	/* Okay, now we've got something to transmit */
	txlin = mulaw[txc];
	if ((ms->flags & AUDIO) && !ms->confmute) {
		/* Handle conferencing on non-clear channel and non-HDLC channels */
		switch(ms->confmode & TOR_CONF_MODE_MASK) {
		case TOR_CONF_NORMAL:
			/* Do nuffin */
			txc = lin2mu[txlin + 32768];
			break;
		case TOR_CONF_MONITOR:	/* Monitor a channel's rx mode */
			/* Add monitored channel */
			txlin += tor_softc[ms->confn].rxlin;
			/* Clip */
			if (txlin > 32767) txlin = 32767;
			if (txlin < -32767) txlin = 32767;
			txc = lin2mu[txlin + 32768];
			break;
		case TOR_CONF_MONITORTX: /* Monitor a channel's tx mode */
			txlin += tor_softc[ms->confn].txlin;
			/* Clip */
			if (txlin > 32767) txlin = 32767;
			if (txlin < -32767) txlin = 32767;
			txc = lin2mu[txlin + 32768];
			break;
		case TOR_CONF_MONITORBOTH: /* monitor a channel's rx and tx mode */
			txlin += tor_softc[ms->confn].rxlin +
				 tor_softc[ms->confn].txlin;
			/* Clip */
			if (txlin > 32767) txlin = 32767;
			if (txlin < -32767) txlin = 32767;
			txc = lin2mu[txlin + 32768];
			break;
		case TOR_CONF_REALANDPSEUDO:
			/* This strange mode takes the transmit buffer and
				puts it on the conference, minus its last sample,
				then outputs from the conference minus the 
				real channel's last sample. */
			  /* if to talk on conf */
			if (ms->confmode & TOR_CONF_PSEUDO_TALKER) {
				/* Store temp value */
				k = txlin;
				/* Add conf value */
				k += (int)conf_presums[ms->confn];
				if (k > 32767) 	k = 32767;
				if (k < -32767) k = -32767;
				/*  get amount actually added */
				ms->conflast1 = ((short)k) -
					conf_presums[ms->confn];
				/* Really add in new value */
				conf_presums[ms->confn] += ms->conflast1;
			} else ms->conflast1 = 0;
			txlin = 0;
			txc = 0;
			/* fall through to normal conf mode */
		case TOR_CONF_CONF:	/* Normal conference mode */
			if (ms->flags & PSEUDO) /* if pseudo-channel */
			   {
				  /* if to talk on conf */
				if (ms->confmode & TOR_CONF_TALKER) {
					/* Store temp value */
					k = txlin;
					/* Add conf value */
					k += (int)conf_presums[ms->confn];
					if (k > 32767) 	k = 32767;
					if (k < -32767) k = -32767;
					/*  get amount actually added */
					ms->conflast = ((short)k) -
						conf_presums[ms->confn];
					/* Really add in new value */
					conf_presums[ms->confn] += ms->conflast;
				} else ms->conflast = 0;
				txlin = ms->txlin;
				txc = 0;
				break;
		 	   }
			/* fall through */
		case TOR_CONF_CONFMON:	/* Conference monitor mode */
			if (ms->confmode & TOR_CONF_LISTENER) {
				/* Subtract out last sample written to conf */
				txlin -= ms->conflast;
				/* Add in conference */
				txlin += conf_sums[ms->confn];
			}
			/* Clip */
			if (txlin > 32767) txlin = 32767;
			if (txlin < -32767) txlin = 32767;
			txc = lin2mu[txlin + 32768];
			break;
		case TOR_CONF_CONFANN:
		case TOR_CONF_CONFANNMON:
			/* First, add tx buffer to conf */
			txlin += conf_presums[ms->confn];
			/* Clip */
			if (txlin > 32767) txlin = 32767;
			if (txlin < -32767) txlin = -32767;
			conf_presums[ms->confn] = (short)txlin;
			/* Start with silence */
			txlin = 0;
			/* If a listener on the conf... */
			if (ms->confmode & TOR_CONF_LISTENER) {
				/* Subtract last value written */
				txlin -= ms->conflast;
				/* Add in conf */
				txlin += conf_sums[ms->confn];
			}
			/* Clip */
			if (txlin > 32767) txlin = 32767;
			if (txlin < -32767) txlin = -32767;
			txc = lin2mu[txlin + 32768];
			break;
		}
	}
	ms->txlin = txlin;
	/* This is what to send */
	return txc;
}



static inline void torintr_receive(int unit, unsigned char rxc)
{
	/* Handle transmission for a given unit */
	struct tor_softc *ss = &tor_softc[unit];
	/* We transmit data from our master channel */
	struct tor_softc *ms = &tor_softc[ss->master];
	/* Our receive buffer */
	unsigned char *buf;
	/* Linear version of received data */
	int rxlin,k;
#ifdef CONFIG_TORMENTA_NET
	/* SKB for receiving network stuff */
	struct sk_buff *skb=NULL;
#endif	
	int oldbuf;
	int eof=0;
	int abort=0;
	int res;
	
	rxlin = (int) ss->rxlin = mulaw[rxc];
	
#ifdef CONFIG_TORMENTA_ECHOCANCEL
	if (ms->echocancel) {
		rxlin = process_cc(&ms->ec, ms->txlin, abs(ms->txlin), rxlin, abs(rxlin));
		rxc = lin2mu[(int)rxlin + 32768];
	}
#endif

	/* Take the rxc, twiddle it for conferencing if appropriate and put it
	   back */
	if ((ms->flags & AUDIO) && !ms->confmute) {
		switch(ms->confmode & TOR_CONF_MODE_MASK) {
		case TOR_CONF_NORMAL:		/* Normal mode */
			/* Do nothing.  rx goes output */
			break;
		case TOR_CONF_MONITOR:		/* Monitor a channel's rx mode */
			  /* if not a pseudo-channel, ignore */
			if (!(ms->flags & PSEUDO)) break;
			/* Add monitored channel */
			rxlin += tor_softc[ms->confn].rxlin;
			/* Clip */
			if (rxlin > 32767) rxlin = 32767;
			if (rxlin < -32767) rxlin = 32767;
			rxc = lin2mu[rxlin + 32768];
			break;
		case TOR_CONF_MONITORTX:	/* Monitor a channel's tx mode */
			  /* if not a pseudo-channel, ignore */
			if (!(ms->flags & PSEUDO)) break;
			rxlin += tor_softc[ms->confn].rxlin;
			/* Clip */
			if (rxlin > 32767) rxlin = 32767;
			if (rxlin < -32767) rxlin = 32767;
			rxc = lin2mu[rxlin + 32768];
			break;
		case TOR_CONF_MONITORBOTH:	/* Monitor a channel's tx and rx mode */
			  /* if not a pseudo-channel, ignore */
			if (!(ms->flags & PSEUDO)) break;
			rxlin += tor_softc[ms->confn].rxlin +
				 tor_softc[ms->confn].rxlin;
			/* Clip */
			if (rxlin > 32767) rxlin = 32767;
			if (rxlin < -32767) rxlin = 32767;
			rxc = lin2mu[rxlin + 32768];
			break;
		case TOR_CONF_REALANDPSEUDO:
			  /* do normal conf mode processing */
			if (ms->confmode & TOR_CONF_TALKER) {
				/* Store temp value */
				k = rxlin;
				/* Add conf value */
				k += (int)conf_sums[ms->confn];
				if (k > 32767)
					k = 32767;
				if (k < -32767)
					k = -32767;
				/*  get amount actually added */
				ms->conflast = ((short)k) -
					conf_sums[ms->confn];
				/* Really add in new value */
				conf_sums[ms->confn] += ms->conflast;
			} else ms->conflast = 0;
			  /* do the pseudo-channel part processing */
			rxlin = 0;
			if (ms->confmode & TOR_CONF_PSEUDO_LISTENER) {
				/* Subtract out last sample written to conf */
				rxlin -= ms->conflast1;
				/* Add in conference */
				rxlin += conf_sums[ms->confn];
			}
			/* Clip */
			if (rxlin > 32767) rxlin = 32767;
			if (rxlin < -32767) rxlin = 32767;
			rxc = lin2mu[rxlin + 32768];
			break;
		case TOR_CONF_CONF:	/* Normal conference mode */
			if (ms->flags & PSEUDO) /* if a pseudo-channel */
			   {
				if (ms->confmode & TOR_CONF_LISTENER) {
					/* Subtract out last sample written to conf */
					rxlin -= ms->conflast;
					/* Add in conference */
					rxlin += conf_sums[ms->confn];
				}
				/* Clip */
				if (rxlin > 32767) rxlin = 32767;
				if (rxlin < -32767) rxlin = 32767;
				rxc = lin2mu[rxlin + 32768];
				ss->txlin = mulaw[rxc];
				break;
			   }
			/* fall through */
		case TOR_CONF_CONFANN:  /* Conference with announce */
			if (ms->confmode & TOR_CONF_TALKER) {
				/* Store temp value */
				k = rxlin;
				/* Add conf value */
				k += (int)conf_sums[ms->confn];
				if (k > 32767)
					k = 32767;
				if (k < -32767)
					k = -32767;
				/*  get amount actually added */
				ms->conflast = ((short)k) -
					conf_sums[ms->confn];
				/* Really add in new value */
				conf_sums[ms->confn] += ms->conflast;
			} else ms->conflast = 0;
			  /* rxc unmodified */
			break;
		case TOR_CONF_CONFMON:
		case TOR_CONF_CONFANNMON:
			if (ms->confmode & TOR_CONF_TALKER) {
				/* Store temp value */
				k = rxlin;
				/* Subtract last value */
				conf_sums[ms->confn] -= ms->conflast;
				/* Add conf value */
				k += (int)conf_sums[ms->confn];
				if (k > 32767)
					k = 32767;
				if (k < -32767)
					k = -32767;
				/*  get amount actually added */
				ms->conflast = ((short)k) -
					conf_sums[ms->confn];
				/* Really add in new value */
				conf_sums[ms->confn] += ms->conflast;
			} else ms->conflast = 0;
			rxc = lin2mu[(int)conf_prev[ms->confn] + 32768];
			break;
		}
	}
	
	/* Next, figure out if we've got a buffer to receive into */
	if (ms->inreadbuf > -1) {
		/* Read into the current buffer */
		buf = ms->readbuf[ms->inreadbuf];
		if (ms->flags & HDLC) {
			/* Handle HDLC deframing */
			fasthdlc_rx_load_nocheck(&ms->rxhdlc, rxc);
			res = fasthdlc_rx_run(&ms->rxhdlc);
			/* Return immediately if there is nothing to receive */
			if (res & RETURN_EMPTY_FLAG)
				return;
			else if (res & RETURN_COMPLETE_FLAG) {
				/* Only count this if it's a non-empty frame */
				if (ms->readidx[ms->inreadbuf]) {
					if ((ms->flags & FCS) && (ms->infcs != PPP_GOODFCS)) {
#if 0
						printk("Tormenta: Bad FCS on Channel %d\n", ss->master);
#endif
						abort = TOR_EVENT_BADFCS;
					} else
						eof=1;
				}
			} else if (res & RETURN_DISCARD_FLAG) {
				/* This could be someone idling with 
				   "idle" instead of "flag" */
				if (!ms->readidx[ms->inreadbuf])
					return;
				abort = TOR_EVENT_ABORT;
			} else {
				rxc = res;
				ms->infcs = PPP_FCS(ms->infcs, rxc);
				buf[ms->readidx[ms->inreadbuf]++] = rxc;
				/* Pay attention to the possibility of an overrun */
				if (ms->readidx[ms->inreadbuf] >= ms->blocksize) {
					if (!alarms[(unit - 1) / 24]) 
						printk(KERN_WARNING "HDLC Receiver overrun on channel %d (master=%d)\n", unit, ss->master);
					abort=TOR_EVENT_OVERRUN;
					/* Force the HDLC state back to frame-search mode */
					ms->rxhdlc.state = 0;
					ms->rxhdlc.bits = 0;
					ms->readidx[ms->inreadbuf]=0;
				}
			}
		} else {
			buf[ms->readidx[ms->inreadbuf]++] = rxc;
			/* End of frame is decided by block size of 'N' */
			eof = (ms->readidx[ms->inreadbuf] >= ms->blocksize);
		}
		if (eof)  {
			/* Finished with this buffer, try another. */
			oldbuf = ms->inreadbuf;
			ms->infcs = PPP_INITFCS;
			ms->readn[ms->inreadbuf] = ms->readidx[ms->inreadbuf];
#if 0
			printk("EOF, len is %d\n", ms->readn[ms->inreadbuf]);
#endif
			if (ms->flags & NETDEV) {
#ifdef CONFIG_TORMENTA_NET
				/* Our network receiver logic is MUCH
				   different.  We actually only us a single
				   buffer */
				if (ms->readn[ms->inreadbuf] > 1) {
					/* Drop the FCS */
					ms->readn[ms->inreadbuf] -= 2;
					/* Allocate an SKB */
					skb = dev_alloc_skb(ms->readn[ms->inreadbuf]);
					if (skb) {
						/* XXX Get rid of this memcpy XXX */
						memcpy(skb->data, ms->readbuf[ms->inreadbuf], ms->readn[ms->inreadbuf]);
						skb_put(skb, ms->readn[ms->inreadbuf]);
					} else {
						ms->netdev.stats.rx_dropped++;
#if 1
						printk("Memory squeeze, dropped one\n");
#endif
					}
				}
				/* We don't cycle through buffers, just
				   reuse the same one */
				ms->readn[ms->inreadbuf] = 0;
				ms->readidx[ms->inreadbuf] = 0;
#endif				
			} else {
				ms->inreadbuf = (ms->inreadbuf + 1) % ms->numbufs;
				if (ms->inreadbuf == ms->outreadbuf) {
					/* Whoops, we're full, and have no where else
					to store into at the moment.  We'll drop it
					until there's a buffer available */
#if 0
					printk("Out of storage space\n");
#endif
					ms->inreadbuf = -1;
					/* Enable the receiver in case they've got POLICY_WHEN_FULL */
					ms->rxdisable = 0;
				}
				if (ms->outreadbuf < 0) {
					/* Notify a blocked reader that there is data available
					to be read, unless we're waiting for it to be full */
#if 0
					printk("Notifying reader data in block %d\n", oldbuf);
#endif
					ms->outreadbuf = oldbuf;
					wake_up_interruptible(&ms->readbufq);
					wake_up_interruptible(&ms->sel);
					if (ms->iomask & TOR_IOMUX_READ)
						wake_up_interruptible(&ms->eventbufq);
				}
			}
		}
		if (abort) {
			/* Start over reading frame */
			ms->readidx[ms->inreadbuf] = 0;
			ms->infcs = PPP_INITFCS;

			if (ms->flags & NETDEV) {
#ifdef CONFIG_TORMENTA_NET
				ms->netdev.stats.rx_errors++;
				if (abort == TOR_EVENT_OVERRUN)
					ms->netdev.stats.rx_over_errors++;
				if (abort == TOR_EVENT_BADFCS)
					ms->netdev.stats.rx_crc_errors++;
				if (abort == TOR_EVENT_ABORT)
					ms->netdev.stats.rx_frame_errors++;
#endif					
			} else if ((ms->flags & OPEN) && !alarms[(unit - 1) / 24]) 
				   /* Notify the receiver... */
				qevent(ss->master, abort);
#if 0
			printk("torintr_receive: Aborted %d bytes of frame on %d\n", amt, ss->master);
#endif

		}
	}
#ifdef CONFIG_TORMENTA_NET
	if (skb) 
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
		hdlc_netif_rx(&ms->netdev, skb,0);
#else
		hdlc_netif_rx(&ms->netdev, skb);
#endif
#endif
}

/*
 * Interrupt procedure.
 */
static void
torintr(int irq, void *dev_id, struct pt_regs *regs)
{
unsigned char txc,rxc,gottx,abits,bbits,c;
static unsigned short rxword[25],txword[25];
int	i,n,n1,j,k,rxlin;
static	u_int ictr,loopupcnt[2],loopdowncnt[2];

	/* We should be called with interrupts diabled */

	  /* make sure its a real interrupt for us */
	if (!(getctlreg() & 1)) /* if not, just return */
	   {
		return; 
	   }

	  /* set outbit and put int 16 bit bus mode, reset interrupt enable */
	setctlreg(clockvals[syncsrc] | OUTBIT | ENA16);

	/* Do the actual transmit and receive in poopy order */
	for(n1 = 0; n1 < 24; n1++)
	   {
	    n = chseq[n1];
	    gottx = 0;
	    tor_softc[0].maddr[DDATA + datxlt[n]] = txword[n];
	    rxword[n] = tor_softc[0].maddr[DDATA + datxlt[n]]; /* get rx word */
	   }

	setctlreg(clockvals[syncsrc] | OUTBIT); /* clear 16 bit mode */

	  /* zero out the pre-sum accumulators */
	memset(conf_presums,0,sizeof(conf_presums));

	/* Calculate the transmit, go thru all the chans */
	for(n1 = 0; n1 < 24; n1++)
	   {
	    gottx = 0;
	    n = chseq[n1];
	    txword[n] = 0;
	      /* go thru both spans */
	    for(j = 0; j < 2; j++)
	      {
		i = n + (j * 24);  /* calc chan number */
		  /* if no such channel */
		if (i > NTOR) break;
		  /* skip if pseudo */ 
		if (tor_softc[i].flags & PSEUDO) continue;  
		  /* enter the transmit stuff with i being channel number,
			leaving with txc being character to transmit */
		txc = torintr_transmit(i);
		if (!(tor_softc[i].flags & AUDIO))
			txword[n] |= txc << (j * 8); 
		else
			txword[n] |= txgainx[i][txc] << (j * 8); 
	      }
	   }

	  /* do transmit routine for all the pseudo-channels */
	for(i = 1; i < NTOR; i++)
	   {
		  /* skip if not psuedo */
		if (!(tor_softc[i].flags & PSEUDO)) continue; 
		torintr_transmit(i);
	   }
	  /* process all the conf links */
	for(i = 1; i <= NCONF; i++)
	   {
		  /* if we have a destination conf */
		if ((j = conf_links[i].dst))
		   {
			   /* get sum of us and them */
			rxlin = conf_sums[conf_links[i].src] + conf_presums[j];
			   /* clip it..... clip it good.. da da da da da.. da da da da */
			if (rxlin > 32767) rxlin = 32767;
			if (rxlin < -32767) rxlin = -32767;
			   /* put it back into accumulator */
			conf_presums[j] = rxlin;
		   }
	   }			
	  /* copy conf pre_sums into sums (thus initializing sums) */
	for(i = 1; i <= NCONF; i++)
	   {
		conf_prev[i] = conf_sums[i];
		conf_sums[i] = conf_presums[i];
	   }
	  /* do the receive for all chans, both spans */
	for(n1 = 0; n1 < 24; n1++)
	   {
	    n = chseq[n1];
	      /* go thru both spans */
	    for(j = 0; j <= 1; j++)
	      {
		i = n + (j * 24);  /* calc chan number */
		  /* if no such channel */
		if (i > NTOR) break;
		  /* skip if pseudo */ 
		if (tor_softc[i].flags & PSEUDO) continue; 
		   /* get rx byte out of word */ 
		if (!(tor_softc[i].flags & AUDIO))
			rxc = (rxword[n] >> (j * 8)) & 0xff;
		else
			rxc = rxgainx[i][(rxword[n] >> (j * 8)) & 0xff] ;
		torintr_receive(i, rxc);
	      }
	   }
	  /* do receive routine for all the pseudo-channels */
	for(i = 1; i < NTOR; i++)
	   {
		  /* skip if not psuedo */
		if (!(tor_softc[i].flags & PSEUDO)) continue; 
		rxc = 0x7f;
		torintr_receive(i, rxc);
	   }

	  /* bump ictr, and see if its divisible by 8 */
	ictr++;
	if ((ictr & 7) == 0)  /* do stuff every millisecond */
	   {
		/* go thru all the chans */
		for(i = 1; i < NTOR; i++)
		   {
			  /* skip if not open */
	/*		if (!(tor_softc[i].flags & OPEN)) continue; */
			  /* skip if pseudo */ 
			if (tor_softc[i].flags & PSEUDO) continue; 
			  /* if itimer to process, do it */
			if (tor_softc[i].itimer && (!--tor_softc[i].itimer))
				do_itimer(i);
			   /* if otimer to process, do it */
			if (tor_softc[i].otimer && (!--tor_softc[i].otimer))
				do_otimer(i);
		   }
		for(i = 0; i < 2; i++)
		   {
			  /* if maint time, and it's timed out */
			if (mainttimer[i] && (!--mainttimer[i]))
			   {
				t1out(i + 1,0x30,0);  /* stop sending code */
				alarm_notify();
				wake_up_interruptible(&maintq[i]);
			   }
			  /* if alarm timer, and it's timed out */
			if (alarmtimer[i] && (!--alarmtimer[i]))
			   {
				  /* clear recover status */
				alarms[i] &= ~TOR_ALARM_RECOVER;
				t1out(i + 1,0x35,0x10); /* turn off yel */
				alarm_notify();  /* let them know */
			   }
		   }
	   }
	i = ictr & 511;
	if (i < 6)  /* do the signalling stuff */
	   {
		k = (i / 3);  /* get span */
		n = (i % 3);  /* get base */
		abits = t1in(k + 1,0x60 + n);
		bbits = t1in(k + 1,0x63 + n);
		  /* go thru all the chans represented */
		for(j = 0; j < 8; j++)
		   {
			  /* calculate real channel number */
			i = (k * 24) + (n * 8) + j + 1;
			  /* if channel not existant, skip */
			if (i >= NTOR) break;
			  /* skip if not open */
	/*		if (!(tor_softc[i].flags & OPEN)) continue; */
			  /* assemble the sig bits into one byte */
			rxc = 0;
			if (abits & (1 << j)) rxc |= TOR_ABIT;
			if (bbits & (1 << j)) rxc |= TOR_BBIT;
			  /* process it */
			if (tor_softc[i].rxsig != rxc) rxchange(i,rxc);
			tor_softc[i].rxsig = rxc;
		   }
	   }
	i = ictr & 511;
	if ((i == 100) || (i == 101))
	   {
		j = 0;  /* clear this alarm status */
		i -= 100;
		t1out(i + 1,0x20,0xff); 
		c = t1in(i + 1,0x20);  /* get the status */
		  /* detect the code, only if we are not sending one */
		if ((!mainttimer[i]) && (c & 0x80))  /* if loop-up code detected */
		   {
			  /* set into remote loop, if not there already */
			if ((loopupcnt[i]++ > 80) && 
				(maints[i] != TOR_MAINT_REMOTELOOP))
			   {
				t1out(i + 1,0x37,0x9c); /* remote loopback */
				maints[i] = TOR_MAINT_REMOTELOOP;
			   }
		   } else loopupcnt[i] = 0;
		  /* detect the code, only if we are not sending one */
		if ((!mainttimer[i]) && (c & 0x40))  /* if loop-down code detected */
		   {
			  /* if in remote loop, get out of it */
			if ((loopdowncnt[i]++ > 80) &&
				(maints[i] == TOR_MAINT_REMOTELOOP))
			   {
				t1out(i + 1,0x37,0x8c); /* normal */
				maints[i] = TOR_MAINT_NONE;
			   }
		   } else loopdowncnt[i] = 0;
		if (c & 3) /* if red alarm */
		   {
			j |= TOR_ALARM_RED;
		   }
		if (c & 8) /* if blue alarm */
		   {
			j |= TOR_ALARM_BLUE;
		   }
		  /* only consider previous carrier alarm state */
		alarms[i] &= (TOR_ALARM_RED | TOR_ALARM_BLUE | TOR_ALARM_NOTOPEN);
		n = 1; /* set to 1 so will not be in yellow alarm if we dont
			care about open channels */
		  /* if to have yellow alarm if nothing open */
		if (lineconfig[i] & TOR_CONFIG_NOTOPEN)
		   {
			  /* go thru all chans, and count # open */
			for(n = 0,k = (i * 24) + 1; k <= (i * 24) + 24; k++) 
			   {
				if ((tor_softc[k].flags & OPEN) ||
				    (tor_softc[k].flags & NETDEV)) n++;
			   }
			  /* if none open, set alarm condition */
			if (!n) j |= TOR_ALARM_NOTOPEN; 
		   }
		  /* if no more alarms, and we had some */
		if ((!j) && alarms[i])
		   {
			alarmtimer[i] = TOR_ALARMSETTLE_TIME; 
		   }
		if (alarmtimer[i]) j |= TOR_ALARM_RECOVER;
		  /* if going into alarm state, set yellow alarm */
		if ((j) && (!alarms[i])) t1out(i + 1,0x35,0x11);
		if (c & 4) /* if yellow alarm */
			j |= TOR_ALARM_YELLOW;
		if (maints[i] || mainttimer[i]) j |= TOR_ALARM_LOOPBACK;
		alarms[i] = j;
	   }
	alarm_notify();
	if (!(ictr % 8000)) /* even second boundary */
	   {
		  /* do both spans */
		for(i = 1; i <= 2; i++)
		   {
			   /* add this second's BPV count to total one */
			bpvcounts[i - 1] += t1in(i,0x24) + (t1in(i,0x23) << 8);
		   }
	   }
	/* re-evaluate active sync src */
	syncsrc = 0;
	  /* if primary sync specified, see if we can use it */
	if (syncs[0])
	   {
		  /* if no alarms, use it */
		if (!(alarms[syncs[0] - 1] & (TOR_ALARM_RED | TOR_ALARM_BLUE | 
			TOR_ALARM_LOOPBACK))) syncsrc = syncs[0];
	   }
	   /* if we dont have one yet, and there is a secondary, see if we can use it */
	if ((!syncsrc) && (syncs[1]))
	   {
		  /* if no alarms, use it */
		if (!(alarms[syncs[1] - 1] & (TOR_ALARM_RED | TOR_ALARM_BLUE | 
			TOR_ALARM_LOOPBACK))) syncsrc = syncs[1];
	   }
	  /* clear outbit, restore interrupt enable */
	setctlreg(clockvals[syncsrc] | INTENA);

}

/*
** This routine converts from linear to ulaw
**
** Craig Reese: IDA/Supercomputing Research Center
** Joe Campbell: Department of Defense
** 29 September 1989
**
** References:
** 1) CCITT Recommendation G.711  (very difficult to follow)
** 2) "A New Digital Technique for Implementation of Any
**     Continuous PCM Companding Law," Villeret, Michel,
**     et al. 1973 IEEE Int. Conf. on Communications, Vol 1,
**     1973, pg. 11.12-11.17
** 3) MIL-STD-188-113,"Interoperability and Performance Standards
**     for Analog-to_Digital Conversion Techniques,"
**     17 February 1987
**
** Input: Signed 16 bit linear sample
** Output: 8 bit ulaw sample
*/

#define ZEROTRAP    /* turn on the trap as per the MIL-STD */
#define BIAS 0x84   /* define the add-in bias for 16 bit samples */
#define CLIP 32635

static unsigned char
linear2ulaw(short sample)
{
  static int exp_lut[256] = {0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
                             4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
                             5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                             5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
                             7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};
  int sign, exponent, mantissa;
  unsigned char ulawbyte;

  /* Get the sample into sign-magnitude. */
  sign = (sample >> 8) & 0x80;          /* set aside the sign */
  if (sign != 0) sample = -sample;              /* get magnitude */
  if (sample > CLIP) sample = CLIP;             /* clip the magnitude */

  /* Convert from 16 bit linear to ulaw. */
  sample = sample + BIAS;
  exponent = exp_lut[(sample >> 7) & 0xFF];
  mantissa = (sample >> (exponent + 3)) & 0x0F;
  ulawbyte = ~(sign | (exponent << 4) | mantissa);
#ifdef ZEROTRAP
  if (ulawbyte == 0) ulawbyte = 0x02;   /* optional CCITT trap */
#endif

  return(ulawbyte);
}

struct file_operations tor_fops = {
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	owner: THIS_MODULE,
#endif
	llseek: NULL,
	read: tor_read, /* read */
	write: tor_write, /* write */
	poll: tor_poll, /* poll */
	ioctl: tor_ioctl, /* ioctl */
	mmap: NULL,
	open: tor_open, /* open */
	flush: NULL, /* flush */
	release: tor_release, /* release */
	fsync: NULL, /* fsync */
	fasync: NULL, /* fasync */
};

static void init_channel(int i)
{
  register int j;

	memset(&tor_softc[i], 0, sizeof(struct tor_softc));
	tor_softc[i].master = i;
	tor_softc[i].outwritebuf = -1;
	tor_softc[i].inwritebuf = -1;
	tor_softc[i].outreadbuf = -1;
	tor_softc[i].inreadbuf = -1;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	init_waitqueue_head(&tor_softc[i].sel);
	init_waitqueue_head(&tor_softc[i].readbufq);
	init_waitqueue_head(&tor_softc[i].writebufq);
	init_waitqueue_head(&tor_softc[i].eventbufq);
	init_waitqueue_head(&tor_softc[i].txstateq);
#endif
	  /* init the translation tables */
	for(j = 0; j < 256; j++) txgainx[i][j] = 
		rxgainx[i][j] = j;
}

#ifndef MODULE

static int __init set_tor_base(char *str)
{
	base = simple_strtol(str, NULL, 0);
	return 1;
}

__setup("tor=", set_tor_base);

int tor_init(void)
{
	int i;
	for(i = 0; i < NTOR; i++) 
		init_channel(i);
	if ((tor_major = register_chrdev(TOR_MAJOR, "tor", &tor_fops)))
	   {
		printk("unable to get major for tor device\n");
		return -EIO;
	   }
	tor_major = TOR_MAJOR;

	tor_softc[0].maddr = phys_to_virt(base);
	tor_softc[0].irq = irq;
	if (!tor_probe()) {
		printk("No tormenta card found at address 0x%05lx.\n", base);
		unregister_chrdev(tor_major, "tor");
		return -EIO;
	}

	/* Initialize our tone libraries */
	tone_zone_init();

	/* Pre-calculate the HDLC tables */
	fasthdlc_precalc();
	return 0;
}

module_init(tor_init);

#else
int init_module(void)
{
	int i;
	if (!base) {
		printk("Specify address with base=0xNNNNN\n");
		return -EIO;
	}
	memset(tor_softc,0,sizeof(tor_softc));
	for(i = 0; i < NTOR; i++) 
		init_channel(i);

	if ((tor_major = register_chrdev(TOR_MAJOR, "tor", &tor_fops)) == -EBUSY)
	   {
		printk("Unable to get major for tor device\n");
		return -EIO;
	   }
	tor_major = TOR_MAJOR;
	tor_softc[0].maddr = phys_to_virt(base);
	tor_softc[0].irq = irq;
	if (!tor_probe()) {
		printk("No tormenta card found at address 0x%05lx.\n", base);
		unregister_chrdev(tor_major, "tor");
		return -EIO;
	}
	printk(KERN_INFO "Tormenta available on major %d\n",tor_major);
	/* Initialize our tone libraries */
	tone_zone_init();

	/* Pre-calculate the HDLC tables */
	fasthdlc_precalc();
	return 0;
}


void cleanup_module(void)
{
	int x;
	tor_shutdown();
	  /* if started, get ridda irq */
	if (tor_softc[0].flags & STARTED)
		free_irq(irq,NULL);
#ifdef CONFIG_TORMENTA_NET
	  /* Unregister any interfaces as appropriate */
	for (x=1;x<NTOR;x++)
		if (tor_softc[x].flags & NETDEV)
			unregister_hdlc_device(&tor_softc[x].netdev);
#endif			
	unregister_chrdev(tor_major, "tor");
	for (x=0;x<TOR_TONE_ZONE_MAX;x++)
		if (tone_zones[x])
			kfree(tone_zones[x]);
}

#endif


#endif /* NTOR */



